## file ok, last checked: 03.05.2007

## checks for differences in the annotation of an updated R annotation package and the annotation in the annotaiton database table.
checkForDifferentAnnotation <- function(con,chip="hgu133plus2",check.columns=c("GenBank","UniGene","LocusLink"),v=TRUE){
	require(pgUtils)
	require("annaffy")
	annotation <- dbGetResult(dbSendQuery(con,"SELECT * FROM annotation WHERE chip=",rpgsql.format.values(chip)))
	annotation <- as.matrix(annotation)
	New.annotation <- matrix(ncol=(length(check.columns)+1),nrow=nrow(annotation))
	rownames(New.annotation) <- annotation[,"id"]
	New.annotation[,1] <- annotation[,"id"]
	col.names <- c("id")
	if(v){
		cat("Creating the new annotation table\n")
		progress(0,steps=40)
	}
	for(i in 1:length(check.columns)){
		if(check.columns[i]=="GenBank" | check.columns[i]=="Genbank" | check.columns[i]=="gen_bank"){
			col.names <- c(col.names,"gen_bank")
			New.annotation[,(i+1)] <- getText(aafGenBank(New.annotation[,1],chip))
		}
		if(check.columns[i]=="UniGene" | check.columns[i]=="Unigene" | check.columns[i]=="uni_gene"){
			col.names <- c(col.names,"uni_gene")
			New.annotation[,(i+1)] <- getText(aafUniGene(New.annotation[,1],chip))
		}
		if(check.columns[i]=="LocusLink" | check.columns[i]=="Locuslink" | check.columns[i]=="locuslink"){
			col.names <- c(col.names,"locuslink")
			New.annotation[,(i+1)] <- getText(aafLocusLink(New.annotation[,1],chip))
		}
		if(check.columns[i]=="Symbol" | check.columns[i]=="Name" | check.columns[i]=="symbol"){
			col.names <- c(col.names,"symbol")
			New.annotation[,(i+1)] <- getText(aafSymbol(New.annotation[,1],chip))
		}
		if(v){
			progress(i*100/length(check.columns),steps=40)
		}
	}
	if(v){
		progress(100,steps=40)
		cat(" finished\n\n")
	}
	colnames(New.annotation) <- col.names
	c.length <- ncol(New.annotation)
	diff.table.c <- 2 * c.length
	if(v){
		cat("comparing the annotation\n")
		progress(0,steps=40)
		nrow.ann <- nrow(annotation)
	}
	DiffTable <- NULL
	dummy <- NULL
	for(i in 1:nrow.ann){
		if(min(annotation[i,col.names]==New.annotation[i,col.names])==0){
			if(is.null(DiffTable)){
				DiffTable <- matrix(ncol=diff.table.c,nrow=1)
#				cat(annotation[i,col.names],"\n")
#				cat("dimension DiffTable:",dim(DiffTable),"dimension annotation:",dim(annotation),"dimension New.annotation",dim(New.annotation),"\n")
				DiffTable[1,1:c.length] <- annotation[i,col.names]
#				cat(New.annotation[i,col.names],"\n")
				DiffTable[1,(c.length+1):ncol(DiffTable)] <- New.annotation[i,col.names]
#				cat("first\n")
			}
			else{
				dummy <- DiffTable
				DiffTable <- matrix(ncol=diff.table.c,nrow=(nrow(dummy)+1))
				DiffTable[1:nrow(dummy),] <- dummy
				DiffTable[nrow(DiffTable),1:c.length] <- annotation[i,col.names]
				DiffTable[nrow(DiffTable),(c.length+1):ncol(DiffTable)] <- New.annotation[i,col.names]
#				cat("next\n")
			}
		}
		if(v){
			progress(i*100/nrow.ann,steps=40)
		}
	}
	if(v){
		progress(100,steps=40)
		cat(" finished\n\n")
	}
	colnames(DiffTable) <- c(paste("old",col.names),paste("new",col.names))
	DiffTable
}


dbUpdateAnnotation <- function(Con,data=NULL,chip=NULL,date=NULL,do.backup=TRUE,v=TRUE){
	Suggested = c("id","gen_bank","description","uni_gene","locuslink","symbol","chromosome","chromosome_location","chip","add_info")
	if(is.null(data)){
		stop("Please submit the data that should be inserted into the database as a matrix or data.frame!\n Suggested columns are id, gen_bank, description, uni_gene, locuslink, symbol, chromosome, chromosome_location (which are the annotations provided by Bioconductors annotation packages for Affymetrix GeneChips).")
	}
	if(is.null(chip)){
		stop("Submit a identifier for your chip! For Affymetrix GeneChips for example chip=hgu133plus2.\n\n")
	}
	#require(tkfiDB)
	if(is.null(date)){
		date <- "unknown"
	}
	##
	# check if the data has a column named id:
	Colnames.data <- colnames(data)
	if(is.null(Colnames.data)){
		stop("The data frame you submitted does not have column names! Submit a table with column names!")
	}
	if("id" %in% Colnames.data){
	}
	else{
		stop("The submitted data frame must have one column named id containing the unique ids (e.g. Affymetrix pobeset ids).")
	}

	table.list <- dbListTables(Con)
	if( !("annotation" %in% table.list$relname)){
		NewCols <- unique(c(Colnames.data,Suggested))
		if(v){
			cat("creating new annotation table with the following columns:\n")
			for(col in NewCols){
				cat(" ",col,"\n")
			}
		}
		createDBTable(Con,name="annotation",attributes=c(NewCols))
		Colnames.toinsert <- Colnames.data[Colnames.data %in% NewCols]
	}
	else{
		# check which columns from the annotation table can be updated with the submitted data.
		Result <- dbGetResult(dbSendQuery(Con,"SELECT * FROM annotation LIMIT 1"))
		Colnames.annotation <- colnames(Result)
		Colnames.toinsert <- Colnames.data[Colnames.data %in% Colnames.annotation]
		if(v){
			NotInsert <- Colnames.data[!(Colnames.data %in% Colnames.annotation)]
			if(length(NotInsert)>0){
				cat("The following columns from the data table are not available in the annotation table in the database. The information from these columns will not be inserted into the table.\n")
				for(col in NotInsert){
					cat(" ",col,"\n")
				}
			}
		}
	}
	if( !("annotation_version" %in% table.list$relname)){
		if(v){
			cat("creating new annotation_version table...\n")
		}
		createDBTable(Con,name="annotation_version",attributes=c("chip","date","update_date","description"),options=c("UNIQUE","","",""))
	}

	new.data <- data[,Colnames.toinsert]
	new.data <- cbind(new.data,chip=chip)
	rownames(new.data) <- new.data[,"id"]

	##
	# doing backup of the values in the db...
	if(do.backup){
		table <- dbGetResult(dbSendQuery(Con,"SELECT * FROM annotation_version WHERE chip=",rpgsql.format.values(chip)))
		if(nrow(table)>0){
			if(v){
				cat("Doing backup...\n")
			}
			filename <- chip
			if(nrow(table>0)){
				filename <- paste(chip,table[1,"date"],sep="_")
			}
			filename <- paste(filename,".txt",sep="")
			if(v){
				cat(paste("writing backup to file",filename,"\n"))
			}
			Old.data <- dbGetResult(dbSendQuery(Con,"SELECT * FROM annotation WHERE chip=",rpgsql.format.values(chip)))
			write.table(Old.data,filename,sep="\t",quote=FALSE,row.names=FALSE)
		}
		else{
			if(v){
				cat("Nothing to backup...\n")
			}
		}
	}

	IDs <- dbGetResult(dbSendQuery(Con,"SELECT id FROM annotation WHERE chip=",rpgsql.format.values(chip)))[,"id"]
	BM <- rep(FALSE,nrow(new.data))
	if(length(IDs) > 0){
		if(v){
			cat("updating...")
		}
		BM <- new.data[,"id"] %in% IDs
		updateDBTable(Con,name="annotation",where.col="id",new.values=new.data[BM,],old.values=new.data[BM,"id"])
	}

	if(max(!BM)==1){
		if(v){
			cat("inserting...")
		}
		insertIntoTable(Con,name="annotation",data=new.data[!BM,])
	}

	if(nrow(dbGetResult(dbSendQuery(Con,"SELECT * FROM annotation_version WHERE chip=",rpgsql.format.values(chip))))>0){
		updateDBTable(Con,name="annotation_version",where.col="chip",new.values=c(date,date()),old.values=chip,set.col=c("date","update_date"))
	}
	else{
		insertIntoTable(Con,name="annotation_version",attributes=c("chip","date","update_date"),data=c(chip,date,date()))
	}

	if(v){
		cat("finished\n\n")
	}
}


.dbUpdateAnnotation.old <- function(Con,data=NULL,chip=NULL,date=NULL,do.backup=TRUE,v=TRUE){
	require(pgUtils)
	if(is.null(data)){
		stop("Please submit the data that should be inserted into the database as a matrix or data.frame!\n Allowed columns are id, gen_bank, description, uni_gene, locuslink, symbol, chromosome, chromosome_location, add_info ")
	}
	if(is.null(chip)){
		stop("Submit a identifier for your chip! For Affymetrix GeneChips for example chip=hgu133plus2.\n\n")
	}
	#require(tkfiDB)
	if(is.null(date)){
		date <- "unknown"
	}
	table.list <- dbListTables(Con)
	if( !("annotation" %in% table.list$relname)){
		if(v){
			cat("creating new annotation table...\n")
		}
		createDBTable(Con,name="annotation",attributes=c("id","gen_bank","description","uni_gene","locuslink","symbol","chromosome","chromosome_location","chip","add_info"))
	}
	if( !("annotation_version" %in% table.list$relname)){
		if(v){
			cat("creating new annotation_version table...\n")
		}
		createDBTable(Con,name="annotation_version",attributes=c("chip","date","update_date","description"),options=c("UNIQUE","","",""))
	}

	t2 <- dbGetResult(dbSendQuery(Con,"SELECT * FROM annotation_version WHERE chip=",rpgsql.format.values(chip)))
	
	##
	# doing backup of the values in the db...
	if(do.backup){
		table <- dbGetResult(dbSendQuery(Con,"SELECT * FROM annotation WHERE chip=",rpgsql.format.values(chip)))
		if(nrow(table)>0){
			if(v){
				cat("Doing backup...\n")
			}
			filename <- chip
			if(nrow(t2>0)){
				filename <- paste(chip,t2[1,"date"],sep="_")
			}
			filename <- paste(filename,".txt",sep="")
			if(v){
				cat(paste("writing backup to file",filename,"\n"))
			}
			write.table(table,filename,sep="\t",quote=FALSE,row.names=FALSE)
		}
		else{
			if(v){
				cat("Nothing to backup...\n")
			}
		}
	}
	
	#determining the possible columns to be updated...
	possible = c("id","gen_bank","description","uni_gene","locuslink","symbol","chromosome","chromosome_location")
	available <- colnames(data)[multiGrep(colnames(data),pattern=possible,exact.match=TRUE)]
	
	if(sum(available=="id")==0){
		stop("Please submit the IDs for the genes to be annotated (the column MUST be named id)\n\n")
	}
	new.data <- matrix(ncol=(length(available)+1),nrow=length(data[,1]))
	colnames(new.data) <- c(available,"chip")
	new.data[,"chip"] <- chip
	if(v){
		cat("Columns that will be updated:\n")
	}
	for(i in 1:length(available)){
		new.data[,available[i]] <- data[,available[i]]
		if(v){
			cat(paste("\t",available[i],"\n"))
		}
	}
	
	##old
	#new.data <- matrix(ncol=6,nrow=length(data[,1]))
	#colnames(new.data) <- c("id","gen_bank","description","uni_gene","locuslink","chip")
	#new.data[,"id"] <- data[,"id"]
	#new.data[,"gen_bank"] <- data[,"gen_bank"]
	#new.data[,"description"] <- data[,"description"]
	#new.data[,"uni_gene"] <- data[,"uni_gene"]
	#new.data[,"locuslink"] <- data[,"locuslink"]
	#new.data[,"chip"] <- rep(chip,length(new.data[,1]))
	##old
	
	if(nrow(t2)>0){
		if(v){
			cat("updating...")
		}
		updateDBTable(Con,name="annotation",where.col="id",new.values=new.data,old.values=new.data[,"id"])
		updateDBTable(Con,name="annotation_version",where.col="chip",new.values=c(date,date()),old.values=chip,set.col=c("date","update_date"))
		if(v){
			cat("finished\n\n")
		}
	}
	else{
		if(v){
			cat("inserting...")
		}
		insertIntoTable(Con,name="annotation",data=new.data)
		insertIntoTable(Con,name="annotation_version",attributes=c("chip","date","update_date"),data=c(chip,date,date()))
		if(v){
			cat("finished\n\n")
		}
	}
}

.toMartChip <- function(chip){
	rep <- chip
	if(chip=="hgu133plus2"){
		rep <- "hg_u133_plus_2"
	}
	rep
}


getAnnotation <- function(id=NULL,con=NULL,method="local",chip=NULL,v=TRUE,...){
	match.arg(method,c("local","db","mart"))
	if(method=="local"){
		if(v){
			cat("Using local R annotation packages \n")
			stop("not implemented yet...")
		}
	}
	if(method=="db"){
		if(v){
			cat("Using maDB annotation database \n")
			if(is.null(con)){
				stop("You must connect to the database using the dbConnect function and submit the connection object as con attribute! \n\n")
			}
		}
		result <- dbGetAnnotation(con=con,id=id,v=v,chip=chip,...)
	}
	if(method=="mart"){
		if(v){
			cat("Using biomaRt annotation \n")
			if(is.null(con)){
				stop("You must connect to mart using the function martConnect and submit the connection object as con attribute! \n\n")
			}
		}
		require(biomaRt)
		martT <- getGene(mart=con,id=id,array=.toMartChip(chip),...)
		Names <- names(martT@table)
		result <- matrix(ncol=(length(Names)+1),nrow=length(martT@id))
		colnames(result) <- c("id",Names)
		rownames(result) <- martT@id
		result[,"id"] <- martT@id
		for(i in 1:length(Names)){
			result[,Names[i]] <- unlist(martT@table[Names[i]])
		}
	}
	result
}


dbGetAnnotation <- function(Con,id,columns=c("gen_bank","description","uni_gene","locuslink","symbol","chromosome"),chip=NULL,search.col="id",v=TRUE,quickload=FALSE){
	require(pgUtils)
	if(is.null(chip)){
		stop("please submit a the chip for which you need the annotation!\n\n")
	}
	if(v){
		cat("Quering the database for the ids...\n")
	}
	result <- dbGetResult(dbSendQuery(Con,"SELECT * FROM annotation_version WHERE chip=",rpgsql.format.values(chip)))
	if(nrow(result)==0){
		stop("No annotation available for this chip!\n")
	}
	if(v){
		cat(paste("The annotation was created on :",result[1,"date"],"\n"))
		cat(paste("The annotation was inserted into the database on ",result[1,"update_date"],"\n"))
	}
	counter <- 0
	if(v){
		cat("Annotating genes\n")
		progress(0,steps=40)
		l.id <- length(id)
	}
	#cc <- 11
	#if(length(id)>10){
	#	cat("please wait for 10 points:")
	#	cc <- length(id)/10
	#}
	if(length(id)<1000){
		quickload=FALSE
	}
	where <- "id"
	if(search.col=="gen_bank" | search.col=="GenBank" | search.col=="Genbank"){
		where <- "gen_bank"
	}
	if(search.col=="uni_gene" | search.col=="UniGene" | search.col=="Unigene"){
		where <- "uni_gene"
	}
	if(search.col=="LocusLink" | search.col=="locuslink"){
		where <- "locuslink"
	}
	if(search.col=="symbol"){
		where <- "symbol"
	}
	annotation <- NULL
	columns <- c("id",columns)
	if(where=="id" & quickload){
		if(length(id)>100){
			result <- dbGetResult(dbSendQuery(Con,"SELECT * FROM annotation WHERE chip=",rpgsql.format.values(chip)))
		}
		else{
			result <- dbGetResult(dbSendQuery(Con,"SELECT * FROM annotation WHERE chip=",rpgsql.format.values(chip),"AND id IN (",list.to.csv(rpgsql.format.values(id)),")"))
		}
		if(nrow(result)>0){
			temp.result <- as.matrix(result[,columns])
			rownames(temp.result) <- result[,"id"]
			#new.annotation <- matrix(ncol=length(columns),nrow=length(id))
			annotation <- temp.result[id,]
			colnames(annotation) <- columns
		}
	}
	else{
		for(i in 1:length(id)){
			if(v){
				counter <- counter + 1
				progress(counter*100/l.id,steps=40)
			}
			#if(counter > cc){
			#	cat(".")
			#	counter <- 0
			#}
			result <- dbGetResult(dbSendQuery(Con,"SELECT * FROM annotation WHERE chip=",rpgsql.format.values(chip),"AND",where,"=" ,rpgsql.format.values(id[i])))
			if(nrow(result)>0){
				temp.result <- as.matrix(result[,columns])
				if(is.null(annotation)){
					annotation <- temp.result
				}
				else{
					new.annotation <- matrix(ncol=length(columns),nrow=(length(annotation[,1])+nrow(result)))
					new.annotation[1:length(annotation[,1]),] <- annotation
					new.annotation[(length(annotation[,1])+1):length(new.annotation[,1]),] <- temp.result
					colnames(new.annotation) <- columns
					annotation <- new.annotation
				}
			}
		}
	}
	if(v){
		progress(100,steps=40)
		cat("\nfinished.\n")
	}
	annotation
}



