require(Biobase)
require(limma)
.MESSAGE_EEXPRSET_DEPRACATED="class EexprSet is deprecated, use MadbSet instead. All functions and methods of EexprSet can also be applied to MadbSet."
##########################################################################
## class specifying the number of spots per subgrid and the number of grids per array.
##
setClass("Layout",representation(n.grid.row="numeric",n.grid.col="numeric",n.spot.row="numeric",n.spot.col="numeric"),
	prototype=list(n.grid.row=1,n.grid.col=1,n.spot.row=0,n.spot.col=0)
	)



###########################################################################
## base expression matrix. Layout should define the printer layout, samples should be a list of the sample hybridized
## per array (affy) or per color and array pairs gives information about the arrays that belong together (eg red and green from one array)
##
setClass("EexprSet",contains="exprSet",representation(type="character",printer="Layout",samples="list",array.pairs="numeric",weights="matrix",genes="matrix"),
	prototype=list(type="OneColor",printer=new("Layout"),samples=as.list(NULL),array.pairs=as.numeric(NULL),weights=matrix(),genes=matrix())
)

#################
## initialize method for the EexprSet class, used to print the depracated warning message.
setMethod("initialize","EexprSet",function(.Object,...){
	.Deprecated("MadbSet","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
	callNextMethod(.Object,...)
}
)
if( !isGeneric("initialize") )
	setGeneric("initialize", function(.Object)
	standardGeneric("initialize"))


############
## new class extending the ExpressionSet from Bioconductor!
setClass("MadbSet",contains="ExpressionSet",representation(type="character",printer="Layout",samples="list",array.pairs="numeric",weights="matrix",genes="matrix"),
	prototype=list(type="OneColor",printer=new("Layout"),samples=as.list(NULL),array.pairs=as.numeric(NULL),weights=matrix(),genes=matrix())
)

newMadbSet <- function(x=NULL,...){
	if(class(x)=="EexprSet"){
		d <- new("MadbSet",exprs=exprs(x),annotation=annotation(x),samples=x@samples,array.pairs=x@array.pairs,weights=x@weights,genes=x@genes,type=x@type,printer=x@printer)
		return(d)
	}
	if(class(x)=="RGList" | class(x)=="MAList"){
		return(.MadbSetFromLimma(x))
	}
	else{ 
		if(class(x)=="ExpressionSet"){
			d <- as(x,"MadbSet")
			return(d)
		}
		else{
			return(new("MadbSet",...))
		}
	}
}

.MadbSetFromLimma <- function(object){
	if(is.null(object$R)){
		#assuming that the object is a MAList...
		r <- as.matrix(MAtoR(object$M,object$A))
		g <- as.matrix(MAtoG(object$M,object$A))
		oldNames <- colnames(as.matrix(object$M))
		cat("Converting a limma MAList into a MadbSet...\n")
	}
	else{
		r <- as.matrix(object$R)
		g <- as.matrix(object$G)
		oldNames <- colnames(as.matrix(object$R))
		cat("Converting a limma RGList into a MadbSet...\n")
	}
    ## R, G, M or A slots of limma objects do not have colnames, if they represent only 1 array...
    if(is.null(oldNames)){
        ## getting this info from the $targets slot.
        oldNames <- object$targets$FileName
    }
	array.pairs <- matrix(ncol=((dim(r)[2])*2),nrow=1)
	nexpr <- matrix(ncol=(length(array.pairs[1,])),nrow=length(r[,1]))
	weights <- nexpr
	names <- NULL
	for(i in 1:(length(array.pairs[1,])/2)){
		array.pairs[(i+i-1)] <- i
		array.pairs[(i+i)] <- -i
		names <- c(names,paste(oldNames[i],"Red"),paste(oldNames[i],"Green"))
		nexpr[,(i+i-1)] <- r[,i]
		nexpr[,(i+i)] <- g[,i]
		if(!is.null(object$weights)){
			weights[,(i+i-1)] <- object$weights[,i]
			weights[,(i+i)] <- object$weights[,i]
		}
	}
	colnames(nexpr) <- names
	if(!is.null(object$genes)){
		if(nrow(object$genes)!=nrow(nexpr)){
			stop("Number of measured intensities and annotation (from GAL file) do not match!\nPlease reload the data without GAL file!")
		}
		object$genes <- as.matrix(object$genes)
		if(max(colnames(object$genes)=="ID")==1){
			rownames(nexpr) <- object$genes[,"ID"]
		}
		else if(max(colnames(object$genes)=="id")==1){
			rownames(nexpr) <- object$genes[,"id"]
		}
		else if(max(colnames(object$genes)=="Id")==1){
			rownames(nexpr) <- object$genes[,"Id"]
		}
		else{
			if(!is.null(rownames(object$genes))){
				rownames(nexpr) <- rownames(object$genes)
			}
			else{
				rownames(nexpr) <- as.character(1:nrow(nexpr))
			}
		}
	}
	else{
		rownames(nexpr) <- as.character(1:nrow(nexpr))
	}

	## check if rownames of expression matrix are unique
	if(length(unique(rownames(nexpr)))!=nrow(nexpr)){
		warning("row names of the expression matrix are replaced by its indices, since they are not unique. The column idx in the @genes annotation matrix maps rows to the corresponding features.")
		rownames(nexpr) <- as.character(1:nrow(nexpr))
		object$genes <- cbind(idx=rownames(nexpr),object$genes)
	}

	PD <- data.frame(sample=1:ncol(nexpr))
	rownames(PD) <- colnames(nexpr)
	VM <- data.frame(labelDescription="arbitrary numbering")
	rownames(VM) <- "sample"
	ret <- new("MadbSet",exprs=nexpr,phenoData=new("AnnotatedDataFrame",data=PD,varMetadata=VM))
	ret@type <- "TwoColor"
	ret@annotation <- "custom"

	
	# setting the weghts, if present in the original object.
	if(!is.null(object$weights)){
		cat("Setting the weights... a weights of 0 means the gene was flagged, a weights of one means the signal is ok! \n\n")
		ret@weights <- weights
	}
	if(!is.null(object$genes)){
		cat("Inserting available annotation into the slot @genes\n\n")
		ret@genes <- as.matrix(object$genes)
	}
	else{
		dummy <- as.matrix(rownames(nexpr))
		colnames(dummy) <- "ID"
		ret@genes <- dummy
	}
	#setting the array pairs property, so the plotting methods know what columns belong to each other.
	ret@array.pairs <- as.numeric(array.pairs)
	
#	exprs(ret) <- nexpr
	#ret@se.exprs <- object@se.exprs
	#ret@phenoData <- object@phenoData
	#ret@description <- object@description
	#ret@annotation <- object@annotation
	#ret@notes <- object@notes
	#ret@weights <- NULL
	ret

}


EexprSet <- function(x=NULL,...){
	.Deprecated("newMadbSet","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
	if(class(x)=="exprSet"){
		d <- new("EexprSet",exprs=exprs(x),se.exprs=x@se.exprs,phenoData=x@phenoData,annotation=x@annotation,description=x@description,notes=x@notes,...)
	}
	else{
		d <- new("EexprSet",...)
	}
	d
}

if( !isGeneric("toExprSet") )
	setGeneric("toExprSet", function(object)
	standardGeneric("toExprSet"))

#################################################################################################
setMethod("toExprSet","EexprSet",
	function(object){
		.Deprecated("as",package="maDB","usage of toExprSet is deprecated, use as and MadbSet, ExpressionSet instead.")
		ret <- new("exprSet")
		exprs(ret) <- exprs(object)
		ret@se.exprs <- object@se.exprs
		ret@phenoData <- object@phenoData
		ret@description <- object@description
		ret@annotation <- object@annotation
		ret@notes <- object@notes
		#ret@weights <- NULL
		ret
	})

###############################
# new
if( !isGeneric("getArrays") )
	setGeneric("getArrays", function(object,...)
	standardGeneric("getArrays"))

##################################################################################################
setMethod("getArrays","EexprSet",
	function(object,...){
		.Deprecated("getArrays","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
		array.names <- colnames(exprs(object))
		bm <- rep(FALSE,length(array.names))
		bm[myGrep(array.names,pattern="Green")] <- TRUE
		if(max(bm)==1){
			array.names <- array.names[bm]
			array.names <- unlist(strsplit(array.names," Green"))
		}
		array.names
	})

##################################################################################################
setMethod("getArrays","MadbSet",
	function(object,...){
		array.names <- sampleNames(object)
		bm <- rep(FALSE,length(array.names))
		bm[myGrep(array.names,pattern="Green")] <- TRUE
		if(max(bm)==1){
			array.names <- array.names[bm]
			array.names <- unlist(strsplit(array.names," Green"))
		}
		array.names
	})


##################################################################################################
if( !isGeneric("getSignalChannels") )
	setGeneric("getSignalChannels", function(object,...)
	standardGeneric("getSignalChannels"))

##################################################################################################
setMethod("getSignalChannels","EexprSet",
	function(object,...){
		.Deprecated("getSignalChannels","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
		array.names <- getArrays(object)
		the.list <- list()
		if(length(object@array.pairs)>0){
			#array.names <- unlist(array.names)
			cat("Got two color micro arrays...\n")
			#for(i in 1:length(array.names)){
			#	cat("-",array.names[i],"\n")
			#}
			sign.ch.names <- colnames(exprs(object))
			#for(i in 1:length(sign.ch.names)){
			#	cat("*",sign.ch.names[i],"\n")
			#	
			#}
			#stop("not implemented yet...\n")
			
			##
			# ok, thats a bad hack, if the user does not transform a limma object into a EexprSet using the
			# toEexprSet function this should not really work... anyway... let's hope ;)
			for(i in 1:length(array.names)){
				the.list <- c(the.list,list(new("SignalChannel",color="Cy5",array.index=i,array.name=array.names[i])))
				the.list <- c(the.list,list(new("SignalChannel",color="Cy3",array.index=i,array.name=array.names[i])))
			}
		}
		else{
			cat("Got one color micro arrays...\n")
			for(i in 1:length(array.names)){
				the.list <- c(the.list,list(new("SignalChannel",color="Cy",array.index=i,array.name=array.names[i])))
			}
		}
		the.list
	})

#
###########################

##################################################################################################
setMethod("getSignalChannels","MadbSet",
	function(object,...){
		array.names <- getArrays(object)
		the.list <- list()
		if(length(object@array.pairs)>0){
			cat("Got two color micro arrays...\n")
			sign.ch.names <- sampleNames(object)
			##
			# ok, thats a bad hack, if the user does not transform a limma object into a EexprSet using the
			# toEexprSet function this should not really work... anyway... let's hope ;)
			for(i in 1:length(array.names)){
				the.list <- c(the.list,list(new("SignalChannel",color="Cy5",array.index=i,array.name=array.names[i])))
				the.list <- c(the.list,list(new("SignalChannel",color="Cy3",array.index=i,array.name=array.names[i])))
			}
		}
		else{
			cat("Got one color micro arrays...\n")
			for(i in 1:length(array.names)){
				the.list <- c(the.list,list(new("SignalChannel",color="Cy",array.index=i,array.name=array.names[i])))
			}
		}
		the.list
	})

#
###########################



##################################################################################################
if( !isGeneric("getM") )
	setGeneric("getM", function(object,r=NULL,g=NULL,remove.flagged=FALSE)
	standardGeneric("getM"))

##################################################################################################
setMethod("getM","EexprSet",
	function(object,r=NULL,g=NULL,remove.flagged=FALSE){
		.Deprecated("getM","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
		Values <- exprs(object)
		if(remove.flagged){
			weights <- getWeights(object)
		}
		if(!.is.log(Values)){
			Values <- log2(Values)
		}
		if(!is.null(r) & !is.null(g)){
			newnames <- NULL
			M <- matrix(ncol=length(r),nrow=length(Values[,1]))
			for(i in 1:length(r)){
				if(length(unique(replaceAll(c(colnames(Values)[r[i]],colnames(Values)[g[i]]),old=c(" Red"," Green"),new=c("",""))))==1){
					newnames <- c(newnames,unique(replaceAll(c(colnames(Values)[r[i]],colnames(Values)[g[i]]),old=c(" Red"," Green"),new=c("",""))))
				}
				else{
					newnames <- c(newnames,paste(colnames(Values)[r[i]],"vs",colnames(Values)[g[i]]))
				}
				M[,i] <- Values[,r[i]] - Values[,g[i]]
				if(remove.flagged){
					M[weights[,r[i]]==0 | weights[,g[i]]==0,i] <- NA 
				}
			}
#			if(length(r)>0 & length(r)=length(g)){
#				M <- matrix(ncol=length(r),nrow=length(Values[,1]))
#				newnames <- NULL
#				for(i in 1:length(r)){
#					M[,i] <- Values[,r[]]
#				}
#			}
#			else{
#				M <- matrix(ncol=1,nrow=length(Values[,1]))
#				newnames <- paste(colnames(Values)[r],",",colnames(Values)[g])
#				M[,1] <- Values[,r] - Values[,g]
#			}
		}
		else{
		if(is.null(object@array.pairs) | length(object@array.pairs) != dim(Values)[2]){
			stop("Please define the array.pairs. array.pairs should be a vector with the same length as you have chips (or with two color arrays: 2*number of arrays). The chips that should be compared should have the same number, but different signs (+ for red, - for green)\n\n example: you have four affy chips, first chip has a treatment sample the second the according control sample, the same for the third and fourth chip: (<object>@array.pairs <- c(1,-1,2,-2))\n\n ")
		}
		pairs <- unique(abs(object@array.pairs))
		M <- matrix(ncol=length(pairs),nrow=length(Values[,1]))
		if(remove.flagged){
			weights <- getWeights(object)
		}
		names <- colnames(Values)
		newnames <- NULL
		for(i in 1:length(pairs)){
			bm <- abs(object@array.pairs)==pairs[i]
			current <- object@array.pairs[bm]
			m <- Values[,bm]
			if(remove.flagged){
				m[weights[,bm][,1]==0 | weights[,bm][,2]==0,1] <- NA
			}
			M[,i] <- sign(current[1])*m[,1] + sign(current[2])*m[,2]
			# setting the new colnames...
			dummies <- names[bm]
			if(length(unique(replaceAll(dummies,old=c(" Red"," Green"),new=c("",""))))==1){
				dummy <- unique(replaceAll(dummies,old=c(" Red"," Green"),new=c("","")))
			}
			else{
				if(current[1]<0){
					dummy <- paste(dummies[2],"vs",dummies[1])
				}
				else{
					dummy <- paste(dummies[1],"vs",dummies[2])
				}
			}
			newnames <- c(newnames,dummy)
		}
		}
		if(is.null(dim(M))){
			names(M) <- rownames(Values)
		}
		else{
			colnames(M) <- newnames
			rownames(M) <- rownames(Values)
		}
		M
	})


##################################################################################################
setMethod("getM","MadbSet",
	function(object,r=NULL,g=NULL,remove.flagged=FALSE){
		Values <- exprs(object)
		if(remove.flagged){
			weights <- getWeights(object)
		}
		if(!.is.log(Values)){
			Values <- log2(Values)
		}
		CN <- colnames(Values)
		names(CN) <- CN
		if(!is.null(r) & !is.null(g)){
			newnames <- NULL
			M <- matrix(ncol=length(r),nrow=length(Values[,1]))
			for(i in 1:length(r)){
				if(length(unique(replaceAll(c(CN[r[i]],CN[g[i]]),old=c(" Red"," Green"),new=c("",""))))==1){
					newnames <- c(newnames,unique(replaceAll(c(CN[r[i]],CN[g[i]]),old=c(" Red"," Green"),new=c("",""))))
				}
				else{
					newnames <- c(newnames,paste(CN[r[i]],"vs",CN[g[i]]))
				}
				M[,i] <- Values[,r[i]] - Values[,g[i]]
				if(remove.flagged){
					M[weights[,r[i]]==0 | weights[,g[i]]==0,i] <- NA 
				}
			}
		}
		else{
		if(is.null(object@array.pairs) | length(object@array.pairs) != dim(Values)[2]){
			stop("Please define the array.pairs. array.pairs should be a vector with the same length as you have chips (or with two color arrays: 2*number of arrays). The chips that should be compared should have the same number, but different signs (+ for red, - for green)\n\n example: you have four affy chips, first chip has a treatment sample the second the according control sample, the same for the third and fourth chip: (<object>@array.pairs <- c(1,-1,2,-2))\n\n ")
		}
		pairs <- unique(abs(object@array.pairs))
		M <- matrix(ncol=length(pairs),nrow=length(Values[,1]))
		if(remove.flagged){
			weights <- getWeights(object)
		}
		names <- colnames(Values)
		newnames <- NULL
		for(i in 1:length(pairs)){
			bm <- abs(object@array.pairs)==pairs[i]
			current <- object@array.pairs[bm]
			m <- Values[,bm]
			if(remove.flagged){
				m[weights[,bm][,1]==0 | weights[,bm][,2]==0,1] <- NA
			}
			M[,i] <- sign(current[1])*m[,1] + sign(current[2])*m[,2]
			# setting the new colnames...
			dummies <- names[bm]
			if(length(unique(replaceAll(dummies,old=c(" Red"," Green"),new=c("",""))))==1){
				dummy <- unique(replaceAll(dummies,old=c(" Red"," Green"),new=c("","")))
			}
			else{
				if(current[1]<0){
					dummy <- paste(dummies[2],"vs",dummies[1])
				}
				else{
					dummy <- paste(dummies[1],"vs",dummies[2])
				}
			}
			newnames <- c(newnames,dummy)
		}
		}
		if(is.null(dim(M))){
			names(M) <- rownames(Values)
		}
		else{
			colnames(M) <- newnames
			rownames(M) <- rownames(Values)
		}
		M
	})



##################################################################################################
if( !isGeneric("getA") )
	setGeneric("getA", function(object,r=NULL,g=NULL,remove.flagged=FALSE)
	standardGeneric("getA"))

##################################################################################################
setMethod("getA","EexprSet",
	function(object,r=NULL,g=NULL,remove.flagged=FALSE){
		.Deprecated("getA","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
		Values <- exprs(object)
		if(remove.flagged){
			weights <- getWeights(object)
		}
		if(!.is.log(Values)){
			Values <- log2(Values)
		}
		if(!is.null(r) & !is.null(g)){
			newnames <- NULL
			A <- matrix(ncol=length(r),nrow=length(Values[,1]))
			for(i in 1:length(r)){
				if(length(unique(replaceAll(c(colnames(Values)[r[i]],colnames(Values)[g[i]]),old=c(" Red"," Green"),new=c("",""))))==1){
					newnames <- c(newnames,unique(replaceAll(c(colnames(Values)[r[i]],colnames(Values)[g[i]]),old=c(" Red"," Green"),new=c("",""))))
				}
				else{
					newnames <- c(newnames,paste(colnames(Values)[r[i]],"vs",colnames(Values)[g[i]]))
				}
				A[,i] <- 0.5 * (Values[,r[i]] + Values[,g[i]])
				#newnames <- c(newnames,paste(colnames(Values)[r[i]],"vs",colnames(Values)[g[i]]))
				if(remove.flagged){
					A[weights[,r[i]]==0 | weights[,g[i]]==0,i] <- NA
				}
			}
			#newnames <- paste(colnames(Values)[r],",",colnames(Values)[g])
			#A <- matrix(ncol=1,nrow=length(Values[,1]))
			#A[,1] <- 0.5 * (Values[,r] + Values[,g])
		}
		else{
		if(is.null(object@array.pairs) | length(object@array.pairs) != dim(Values)[2]){
			stop("Please define the array.pairs. array.pairs should be a vector with the same length as you have chips (or with two color arrays: 2*number of arrays). The chips that should be compared should have the same number, but different signs (+ for red, - for green)\n\n example: you have four affy chips, first chip has a treatment sample the second the according control sample, the same for the third and fourth chip: (<object>@array.pairs <- c(1,-1,2,-2))\n\n ")
		}

		pairs <- unique(abs(object@array.pairs))
		A <- matrix(ncol=length(pairs),nrow=length(Values[,1]))
		if(remove.flagged){
			weights <- getWeights(object)
		}
		names <- colnames(Values)
		newnames <- NULL
		for(i in 1:length(pairs)){
			bm <- abs(object@array.pairs)==pairs[i]
			current <- object@array.pairs[bm]
			m <- Values[,bm]
			if(remove.flagged){
				m[weights[,bm][,1]==0 | weights[,bm][,2]==0,1] <- NA
			}
			A[,i] <- 0.5 * (m[,1] + m[,2])
			# setting the new colnames...
			dummies <- names[bm]
			if(length(unique(replaceAll(dummies,old=c(" Red"," Green"),new=c("",""))))==1){
				dummy <- unique(replaceAll(dummies,old=c(" Red"," Green"),new=c("","")))
			}
			else{
				if(current[1]<0){
					dummy <- paste(dummies[2],"vs",dummies[1])
				}
				else{
					dummy <- paste(dummies[1],"vs",dummies[2])
				}
			}
			newnames <- c(newnames,dummy)
		}
		}
		if(is.null(dim(A))){
			names(A) <- rownames(Values)
		}
		else{
			colnames(A) <- newnames
			rownames(A) <- rownames(Values)
		}
		A
	})


##################################################################################################
setMethod("getA","MadbSet",
	function(object,r=NULL,g=NULL,remove.flagged=FALSE){
		Values <- exprs(object)
		if(remove.flagged){
			weights <- getWeights(object)
		}
		if(!.is.log(Values)){
			Values <- log2(Values)
		}
		CN <- colnames(Values)
		names(CN) <- CN
		if(!is.null(r) & !is.null(g)){
			newnames <- NULL
			A <- matrix(ncol=length(r),nrow=length(Values[,1]))
			for(i in 1:length(r)){
				if(length(unique(replaceAll(c(CN[r[i]],CN[g[i]]),old=c(" Red"," Green"),new=c("",""))))==1){
					newnames <- c(newnames,unique(replaceAll(c(CN[r[i]],CN[g[i]]),old=c(" Red"," Green"),new=c("",""))))
				}
				else{
					newnames <- c(newnames,paste(CN[r[i]],"vs",CN[g[i]]))
				}
				A[,i] <- 0.5 * (Values[,r[i]] + Values[,g[i]])
				#newnames <- c(newnames,paste(colnames(Values)[r[i]],"vs",colnames(Values)[g[i]]))
				if(remove.flagged){
					A[weights[,r[i]]==0 | weights[,g[i]]==0,i] <- NA
				}
			}
			#newnames <- paste(colnames(Values)[r],",",colnames(Values)[g])
			#A <- matrix(ncol=1,nrow=length(Values[,1]))
			#A[,1] <- 0.5 * (Values[,r] + Values[,g])
		}
		else{
		if(is.null(object@array.pairs) | length(object@array.pairs) != dim(Values)[2]){
			stop("Please define the array.pairs. array.pairs should be a vector with the same length as you have chips (or with two color arrays: 2*number of arrays). The chips that should be compared should have the same number, but different signs (+ for red, - for green)\n\n example: you have four affy chips, first chip has a treatment sample the second the according control sample, the same for the third and fourth chip: (<object>@array.pairs <- c(1,-1,2,-2))\n\n ")
		}

		pairs <- unique(abs(object@array.pairs))
		A <- matrix(ncol=length(pairs),nrow=length(Values[,1]))
		if(remove.flagged){
			weights <- getWeights(object)
		}
		names <- colnames(Values)
		newnames <- NULL
		for(i in 1:length(pairs)){
			bm <- abs(object@array.pairs)==pairs[i]
			current <- object@array.pairs[bm]
			m <- Values[,bm]
			if(remove.flagged){
				m[weights[,bm][,1]==0 | weights[,bm][,2]==0,1] <- NA
			}
			A[,i] <- 0.5 * (m[,1] + m[,2])
			# setting the new colnames...
			dummies <- names[bm]
			if(length(unique(replaceAll(dummies,old=c(" Red"," Green"),new=c("",""))))==1){
				dummy <- unique(replaceAll(dummies,old=c(" Red"," Green"),new=c("","")))
			}
			else{
				if(current[1]<0){
					dummy <- paste(dummies[2],"vs",dummies[1])
				}
				else{
					dummy <- paste(dummies[1],"vs",dummies[2])
				}
			}
			newnames <- c(newnames,dummy)
		}
		}
		if(is.null(dim(A))){
			names(A) <- rownames(Values)
		}
		else{
			colnames(A) <- newnames
			rownames(A) <- rownames(Values)
		}
		A
	})





##################################################################################################
if( !isGeneric("writeTable") )
	setGeneric("writeTable", function(object,file=NULL,mas5calls=NULL,ids=NULL,include.annotation=TRUE,...)
	standardGeneric("writeTable"))

##################################################################################################
setMethod("writeTable","EexprSet",
	function(object,file=NULL,mas5calls=NULL,ids=NULL,include.annotation=TRUE,...){
		.Deprecated("writeTable","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
		if(object@type=="OneColor"){
			cat("got a single channel array...\n")
			require(annaffy)
			do.it <- FALSE
			if(!is.null(ids)){
				ids <- rownames(exprs(object)[ids,])
				do.it <- TRUE
			}
			else{
				ids <- rownames(exprs(object))
			}
			samples <- colnames(exprs(object))
			ann.cols = 1
			if(include.annotation){
				ann.cols = 6
			}
			# what if the annotation package is not installed or avalable?
			if(is.null(mas5calls)){
				result <- matrix(ncol=(length(samples)+ann.cols),nrow=length(ids))
				
				cat("annotating the probes...")
				result[,1] <- ids
				if(include.annotation){
					if(is.null(tryCatch(dummy <- aafGenBank(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to GenBank IDs!")
					}
					else{
						result[,2] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafUniGene(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to UniGene IDs!")
					}
					else{
						result[,3] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafDescription(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to Gene descriptions!")
					}
					else{
						result[,4] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafLocusLink(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to LocusLink IDs!")
					}
					else{
						result[,5] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafSymbol(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to official gene names!")
					}
					else{
						result[,6] <- getText(dummy)
					}					
					colnames(result) <- c("Id","GenBank","UniGene","Description","LocusLink","Symbol",samples)
				}
				else{
					colnames(result) <- c("Id",samples)
				}
				cat("finished\n")
				cat("writing data...")
				if(do.it){
					result[,(ann.cols+1):length(result[1,])] <- exprs(object)[ids,]
				}
				else{
					result[,(ann.cols+1):length(result[1,])] <- exprs(object)
				}
				write.table(result,file,sep="\t",row.names=FALSE,quote=FALSE)
				cat("finished\n\n")
			}
			else{
				result <- matrix(ncol=(2*length(samples)+ann.cols),nrow=length(ids))
				
				cat("annotating the probes...")
				result[,1] <- ids
				#result[,1] <- ids
				if(include.annotation){
					if(is.null(tryCatch(dummy <- aafGenBank(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to GenBank IDs!")
					}
					else{
						result[,2] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafUniGene(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to UniGene IDs!")
					}
					else{
						result[,3] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafDescription(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to Gene descriptions!")
					}
					else{
						result[,4] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafLocusLink(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to LocusLink IDs!")
					}
					else{
						result[,5] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafSymbol(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to official gene names!")
					}
					else{
						result[,6] <- getText(dummy)
					}					
					#colnames(result) <- c("ID","GenBank","UniGene","Description","LocusLink",samples)
					cnames <- c("Id","GenBank","UniGene","Description","LocusLink","Symbol")
				}
				else{
					#colnames(result) <- c("ID",samples)
					cnames <- c("Id")
				}
				#result[,2] <- getText(aafGenBank(ids,chip=object@annotation))
				#result[,3] <- getText(aafUniGene(ids,chip=object@annotation))
				#result[,4] <- getText(aafDescription(ids,chip=object@annotation))
				#result[,5] <- getText(aafLocusLink(ids,chip=object@annotation))
			
				cat("finished\n")
				cat("writing data...")
				index <- ann.cols
				for(i in 1:length(samples)){
					index <- index+1
					result[,index] <- exprs(object)[,samples[i]]
					index <- index+1
					result[,index] <- exprs(mas5calls)[,samples[i]]
					cnames <- c(cnames,c(samples[i],paste("Present call",samples[i])))
				}
				colnames(result) <- cnames
				write.table(result,file,sep="\t",row.names=FALSE,quote=FALSE)
				cat("finished\n\n")
			}
		}
	}
)


##################################################################################################
setMethod("writeTable","MadbSet",
	function(object,file=NULL,mas5calls=NULL,ids=NULL,include.annotation=TRUE,...){
		if(object@type=="OneColor"){
			cat("got a single channel array...\n")
			require(annaffy)
			do.it <- FALSE
			if(!is.null(ids)){
				ids <- rownames(exprs(object)[ids,])
				do.it <- TRUE
			}
			else{
				ids <- rownames(exprs(object))
			}
			samples <- colnames(exprs(object))
			ann.cols = 1
			if(include.annotation){
				ann.cols = 6
			}
			# what if the annotation package is not installed or avalable?
			if(is.null(mas5calls)){
				result <- matrix(ncol=(length(samples)+ann.cols),nrow=length(ids))
				
				cat("annotating the probes...")
				result[,1] <- ids
				if(include.annotation){
					if(is.null(tryCatch(dummy <- aafGenBank(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to GenBank IDs!")
					}
					else{
						result[,2] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafUniGene(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to UniGene IDs!")
					}
					else{
						result[,3] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafDescription(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to Gene descriptions!")
					}
					else{
						result[,4] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafLocusLink(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to LocusLink IDs!")
					}
					else{
						result[,5] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafSymbol(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to official gene names!")
					}
					else{
						result[,6] <- getText(dummy)
					}					
					colnames(result) <- c("Id","GenBank","UniGene","Description","LocusLink","Symbol",samples)
				}
				else{
					colnames(result) <- c("Id",samples)
				}
				cat("finished\n")
				cat("writing data...")
				if(do.it){
					result[,(ann.cols+1):length(result[1,])] <- exprs(object)[ids,]
				}
				else{
					result[,(ann.cols+1):length(result[1,])] <- exprs(object)
				}
				write.table(result,file,sep="\t",row.names=FALSE,quote=FALSE)
				cat("finished\n\n")
			}
			else{
				result <- matrix(ncol=(2*length(samples)+ann.cols),nrow=length(ids))
				
				cat("annotating the probes...")
				result[,1] <- ids
				#result[,1] <- ids
				if(include.annotation){
					if(is.null(tryCatch(dummy <- aafGenBank(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to GenBank IDs!")
					}
					else{
						result[,2] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafUniGene(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to UniGene IDs!")
					}
					else{
						result[,3] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafDescription(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to Gene descriptions!")
					}
					else{
						result[,4] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafLocusLink(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to LocusLink IDs!")
					}
					else{
						result[,5] <- getText(dummy)
					}
					if(is.null(tryCatch(dummy <- aafSymbol(ids,chip=object@annotation),error=function(e){NULL}))){
						warning("Probesets can not be annotated to official gene names!")
					}
					else{
						result[,6] <- getText(dummy)
					}					
					#colnames(result) <- c("ID","GenBank","UniGene","Description","LocusLink",samples)
					cnames <- c("Id","GenBank","UniGene","Description","LocusLink","Symbol")
				}
				else{
					#colnames(result) <- c("ID",samples)
					cnames <- c("Id")
				}
				#result[,2] <- getText(aafGenBank(ids,chip=object@annotation))
				#result[,3] <- getText(aafUniGene(ids,chip=object@annotation))
				#result[,4] <- getText(aafDescription(ids,chip=object@annotation))
				#result[,5] <- getText(aafLocusLink(ids,chip=object@annotation))
			
				cat("finished\n")
				cat("writing data...")
				index <- ann.cols
				for(i in 1:length(samples)){
					index <- index+1
					result[,index] <- exprs(object)[,samples[i]]
					index <- index+1
					result[,index] <- exprs(mas5calls)[,samples[i]]
					cnames <- c(cnames,c(samples[i],paste("Present call",samples[i])))
				}
				colnames(result) <- cnames
				write.table(result,file,sep="\t",row.names=FALSE,quote=FALSE)
				cat("finished\n\n")
			}
		}
	}
)



###########################################
## simple helper function...
##
.is.log <- function(data,maxi=100){
	data <- data[!is.na(data)]
	data <- data[!is.infinite(data)]
#	cat("Try to check wheter or not the values are in log scale...")
	if(max(data>maxi)){
		log.values = FALSE
#		cat("signals are not in log2 scale, transforming...\n")
	}
	else{
		log.values = TRUE
#		cat("signals are already in log2 scale \n")
	}
	log.values
}

##################################################################################################
if(!isGeneric("getWeights"))
	setGeneric("getWeights", function(object,array.pairs=NULL)
	standardGeneric("getWeights"))

##################################################################################################
setMethod("getWeights","EexprSet",
	function(object,array.pairs=NULL){
		.Deprecated("getWeights","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
		if(is.null(object@weights) | dim(exprs(object))[1] != dim(object@weights)[1] | dim(exprs(object))[2] != dim(object@weights)[2]){
			cat("building a dummy weights matrix... \n\n")
			ret <- matrix(ncol=ncol(exprs(object)),nrow=nrow(exprs(object)))
			ret[,] <- 1
			colnames(ret)  <- sampleNames(object)
		}
		else{
			ret <- object@weights
			colnames(ret)  <- sampleNames(object)
		}
		if(!is.null(array.pairs)){
			cat("array pairs were specified, so i have to rearrange the weights according to the submitted array.pairs parameter... \n\n")
			pairs <- unique(abs(array.pairs))
			retu <- matrix(ncol=length(pairs),nrow=dim(ret)[1])
			for(i in 1:length(pairs)){
				bm <- abs(array.pairs)==pairs[i]
				dummy <- ret[,bm]
				retu[,i] <- as.numeric(dummy[,1]==1 & dummy[,2]==1)
			}
			ret <- retu
			colnames(ret) <- sub(x=colnames(exprs(object))[array.pairs>0],pattern="Red",replacement="")
		}
		## changed 10.01.2006, colnames added above
		#colnames(ret) <- sub(x=colnames(exprs(object))[array.pairs>0],pattern="Red",replacement="")
		
		rownames(ret) <- rownames(exprs(object))
		ret
	})

##################################################################################################
setMethod("getWeights","MadbSet",
	function(object,array.pairs=NULL){
		if(is.null(object@weights) | dim(exprs(object))[1] != dim(object@weights)[1] | dim(exprs(object))[2] != dim(object@weights)[2]){
			cat("building a dummy weights matrix... \n\n")
			ret <- matrix(ncol=ncol(exprs(object)),nrow=nrow(exprs(object)))
			ret[,] <- 1
			colnames(ret)  <- sampleNames(object)
		}
		else{
			ret <- object@weights
			colnames(ret)  <- sampleNames(object)
		}
		if(!is.null(array.pairs)){
			cat("array pairs were specified, so i have to rearrange the weights according to the submitted array.pairs parameter... \n\n")
			pairs <- unique(abs(array.pairs))
			retu <- matrix(ncol=length(pairs),nrow=dim(ret)[1])
			for(i in 1:length(pairs)){
				bm <- abs(array.pairs)==pairs[i]
				dummy <- ret[,bm]
				retu[,i] <- as.numeric(dummy[,1]==1 & dummy[,2]==1)
			}
			ret <- retu
			colnames(ret) <- sub(x=colnames(exprs(object))[array.pairs>0],pattern="Red",replacement="")
		}
		## changed 10.01.2006, colnames added above
		#colnames(ret) <- sub(x=colnames(exprs(object))[array.pairs>0],pattern="Red",replacement="")
		
		rownames(ret) <- rownames(exprs(object))
		ret
	})


##################################################################################################
if( !isGeneric("drawMA") )
	setGeneric("drawMA", function(object,type=NULL,array.pairs=NULL,g=NULL,r=NULL,log.values=NULL,one.image=TRUE,draw.flagged=FALSE,use.weights.from=NULL,main=NULL,colramp=NULL,col=NULL,...)
	standardGeneric("drawMA"))

setMethod("drawMA","exprSet",
	function(object,type=NULL,array.pairs=NULL,g=NULL,r=NULL,log.values=NULL,one.image=TRUE,draw.flagged=FALSE,use.weights.from=NULL,main=NULL,colramp=NULL,col=NULL,...){
#		if(is.null(type)){
#			stop("please submit the type of array you are using, (Affymetrix or TwoColor)\n")
#		}
#		if(is.null(array.pairs)){
#			dimension <- length(sampleNames(object))
#			cat("You have not defined what chips should be used as red and what chip as green array \n, trying to define it by myself...\n\n")
#			if(isDecimal(dimension/2)){
#				cat("have a problem, the number of chips is not even... dropping the last one...\n")
#			}
#			array.pairs <- matrix(nrow=1,ncol=floor(dimension))
#			for(i in 1:(floor(length(array.pairs[1,])/2))){
#				array.pairs[(i+i-1)] <- i
#				array.pairs[(i+i)] <- -i
#				cat(array.pairs[(i+i-1)])
#				cat(" ")
#				cat(array.pairs[(i+i)])
#				cat("\n")
#			}
#			#cat(pairs)
#		}
#		drawMA(new("EexprSet",object,type=as.character(type),array.pairs=as.numeric(array.pairs)),draw.flagged=draw.flagged)
	drawMA(EexprSet(object),type=type,array.pairs=array.pairs,g=g,r=r,log.values=log.values,one.image=one.image,draw.flagged=draw.flagged,user.weights.from=use.weights.from,main=main...)
	})
	

##################################################################################################
setMethod("drawMA","EexprSet",
	function(object,type=NULL,array.pairs=NULL,g=NULL,r=NULL,log.values=NULL,one.image=TRUE,draw.flagged=FALSE,use.weights.from=NULL,main=NULL,colramp=NULL,col=NULL,...){
		.Deprecated("getWeights","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
		colors <- 1
		if(!is.null(col)){
			colramp <- NULL
			colors <- col
		}
		if(!is.null(colramp)){
			##
			# drawing density colored MA plots
			if(!require("geneplotter",character.only=TRUE)){
				colramp=NULL
				warning("library geneplotter which is not installed is required to draw a density colored MA plot!\n")
			}
			else{
				require("geneplotter")
			}
		}
		#source("tkfi.R")
		Values <- exprs(object)
		dimension <- dim(Values)
		colnames <- colnames(Values)
		if(is.null(log.values)){
			log.values = .is.log(Values)
		}
		if(!log.values){
			Values <- log2(Values)
		}
#		switch(object@type,
#			Affymetrix={
			if(!is.null(r) & !is.null(g)){
				cat(paste("taking the chip",colnames[g],"as the green color array and the chip",colnames[r],"as.the red one\n"))
				M <- Values[,r] - Values[,g]
				A <- 0.5 * (Values[,r] + Values[,g])
				weights <- getWeights(object)
				if(!is.null(use.weights.from)){
					dummy <- weights[,use.weights.from]
					weights[,] <- 1
					weights[,use.weights.from] <- dummy
				}
				if(is.null(main)){
					main=paste(colnames[r],",",colnames[g])
				}
				excludeSpots <- weights[,r]==0 | weights[,g]==0
				## remove na values!
				excludeSpots <- excludeSpots | is.na(A) | is.na(M) | is.infinite(M) | is.infinite(A)
				if(!is.null(colramp)){
					colors <- 1
					colors <- tryCatch(densCols(A[!excludeSpots],M[!excludeSpots],colramp=colramp),error=function(error){1})
				}
				drawMAPlot(M[!excludeSpots],A[!excludeSpots],main=main,pch=".",col=colors,...)
				if(draw.flagged){
					points(A[excludeSpots],M[excludeSpots],pch=".",col="orange")
				}
			}
			else{
				#drawing all MAPlots
				#suppose the user has defined an array.pairs parameter.
				if(!is.null(array.pairs)){
					object@array.pairs <- array.pairs
				} 
				m <- getM(object)
				a <- getA(object)
				if(one.image){
					par(mfrow=c((floor(sqrt(dim(m)[2]))),(ceiling(sqrt(dim(m)[2])))))
				}
				
				#cat("the dim of weights: ")
				#cat(dim(object@weights))
				#cat("\nthe dim of exprs: ")
				#cat(dim(exprs(object)))
				#cat("\nthe array.pairs: ")
				#cat(object@array.pairs)
				weights <- getWeights(object,array.pairs=object@array.pairs)
				for(i in 1:(dim(m)[2])){
					if(!one.image){
						X11()
					}
					if(is.null(main)){
						mainstring=colnames(m)[i]
					}
					else{
						mainstring=main
					}
					colors <- 1
					toExclude <- weights[,i]==0 | is.na(a[,i]) | is.na(m[,i])
					if(!is.null(colramp)){
						colors <- densCols(a[,i][!toExclude],m[,i][!toExclude],colramp=colramp)
					}
					drawMAPlot(m[,i][!toExclude],a[,i][!toExclude],main=mainstring,pch=".",col=colors,...)
					if(draw.flagged){
						points(a[,i][toExclude],m[,i][toExclude],pch=".",col="orange")
					}
				}
			}
#			},
#			TwoColor={})
	})

##################################################################################################
setMethod("drawMA","ExpressionSet",
	function(object,type=NULL,array.pairs=NULL,g=NULL,r=NULL,log.values=NULL,one.image=TRUE,draw.flagged=FALSE,use.weights.from=NULL,main=NULL,colramp=NULL,col=NULL,...){
	drawMA(newMadbSet(object),type=type,array.pairs=array.pairs,g=g,r=r,log.values=log.values,one.image=one.image,draw.flagged=draw.flagged,user.weights.from=use.weights.from,main=main,colramp=colramp,col=col,...)
})

##################################################################################################
setMethod("drawMA","MadbSet",
	function(object,type=NULL,array.pairs=NULL,g=NULL,r=NULL,log.values=NULL,one.image=TRUE,draw.flagged=FALSE,use.weights.from=NULL,main=NULL,colramp=NULL,col=NULL,...){
		colors <- 1
		if(!is.null(col)){
			colramp <- NULL
			colors <- col
		}
		if(!is.null(colramp)){
			##
			# drawing density colored MA plots
			if(!require("geneplotter",character.only=TRUE)){
				colramp=NULL
				warning("library geneplotter which is not installed is required to draw a density colored MA plot!\n")
			}
			else{
				require("geneplotter")
			}
		}

		## if pch was submitted in ... set the pch here!
		hasPCH=FALSE
		pch="."
## strangely R crashes...
#		args_ <- list(...)
#		if(length(args_)>0){
#			if(!is.null(args_[["pch"]])){
#				hasPCH=TRUE
#				pch=args_[["pch"]]
#			}
#		}

		#source("tkfi.R")
		Values <- exprs(object)
		dimension <- dim(Values)
		colnames <- colnames(Values)

		if(!is.null(r) & class(r)=="character"){
			idx <- grep(colnames(Values),pattern=r,fixed=TRUE)
			if(length(idx)==0){
				stop("No sample",r,"in the dataset! Use method sampleNames to get a list of available (and allowed) samples.\n")
			}
			r <- idx
		}
		if(!is.null(g) & class(g)=="character"){
			idx <- grep(colnames(Values),pattern=g,fixed=TRUE)
			if(length(idx)==0){
				stop("No sample",g,"in the dataset! Use method sampleNames to get a list of available (and allowed) samples.\n")
			}
			g <- idx
		}

		names(colnames) <- colnames(Values)
		if(is.null(log.values)){
			log.values = .is.log(Values)
		}
		if(!log.values){
			Values <- log2(Values)
		}
#		switch(object@type,
#			Affymetrix={
			if(!is.null(r) & !is.null(g)){
				cat(paste("Generating an MA plot of ",colnames[r],"vs",colnames[g],"\n"))
				M <- Values[,r] - Values[,g]
				A <- 0.5 * (Values[,r] + Values[,g])
				weights <- getWeights(object)
				if(!is.null(use.weights.from)){
					dummy <- weights[,use.weights.from]
					weights[,] <- 1
					weights[,use.weights.from] <- dummy
				}
				if(is.null(main)){
					main=paste(colnames[r],"vs",colnames[g])
				}
				excludeSpots <- weights[,r]==0 | weights[,g]==0
				## remove na values!
				excludeSpots <- excludeSpots | is.na(A) | is.na(M) | is.infinite(M) | is.infinite(A)
				if(!is.null(colramp)){
					colors <- 1
					tryCatch(colors <- densCols(A[!excludeSpots],M[!excludeSpots],colramp=colramp),error=function(error){warning("Could not create density dependent color vector!")})
				}
#				if(hasPCH){
#					drawMAPlot(M[!excludeSpots],A[!excludeSpots],main=main,col=colors,...)
#				}
#				else{
					drawMAPlot(M[!excludeSpots],A[!excludeSpots],main=main,pch=pch,col=colors,...)
#				}
				if(draw.flagged){
					points(A[excludeSpots],M[excludeSpots],pch=pch,col="orange")
				}
			}
			else{
				#drawing all MAPlots
				#suppose the user has defined an array.pairs parameter.
				if(!is.null(array.pairs)){
					object@array.pairs <- array.pairs
				} 
				m <- getM(object)
				a <- getA(object)
				if(one.image){
					par(mfrow=c((floor(sqrt(dim(m)[2]))),(ceiling(sqrt(dim(m)[2])))))
				}
				
				weights <- getWeights(object,array.pairs=object@array.pairs)
				for(i in 1:(dim(m)[2])){
					if(!one.image){
						X11()
					}
					if(is.null(main)){
						mainstring=colnames(m)[i]
					}
					else{
						mainstring=main
					}
					colors <- 1
					toExclude <- weights[,i]==0 | is.na(a[,i]) | is.na(m[,i])
					if(!is.null(colramp)){
						colors <- densCols(a[,i][!toExclude],m[,i][!toExclude],colramp=colramp)
					}
#					if(hasPCH){
#						drawMAPlot(m[,i][!toExclude],a[,i][!toExclude],main=mainstring,col=colors,...)
#					}
#					else{
#						drawMAPlot(m[,i][!toExclude],a[,i][!toExclude],main=mainstring,pch=pch,col=colors,...)
#					}
					if(draw.flagged){
						points(a[,i][toExclude],m[,i][toExclude],pch=pch,col="orange")
					}
				}
			}
#			},
#			TwoColor={})
	})



##################################################################################################
if( !isGeneric("toEexprSet") )
	setGeneric("toEexprSet", function(object)
	standardGeneric("toEexprSet"))

##################################################################################################
setMethod("toEexprSet","RGList",
	function(object){
		ret <- createEexprSetFromLimma(object)
		ret
	})

##################################################################################################
setMethod("toEexprSet","MAList",
	function(object){
		ret <- createEexprSetFromLimma(object)
		ret
	})


##################################################################################################
createEexprSetFromLimma <- function(object){
	.Deprecated("newMadbSet","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
	return (NULL)
}

##################################################################################################
if( !isGeneric("drawHistogram") )
	setGeneric("drawHistogram", function(object,index=NULL,type=NULL,draw.pairs=FALSE,nr.breaks=250,draw.legend=FALSE,col=NULL,...)
	standardGeneric("drawHistogram"))

##################################################################################################
setMethod("drawHistogram","EexprSet",
	function(object,index=NULL,type=NULL,draw.pairs=FALSE,nr.breaks=250,draw.legend=FALSE,col=NULL,...){
		.Deprecated("drawHistogram","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
		Values <- exprs(object)
		if(!.is.log(Values)){
			Values <- log2(Values)
			#cat("dim values",dim(Values),"\n")
		}
		
		names <- colnames(exprs(object))
		weights <- getWeights(object)
		by <- max(Values[!is.na(Values) & !is.infinite(Values)])/nr.breaks
		breaks <- seq(min(0,Values[!is.na(Values) & !is.infinite(Values)]),(max(Values[!is.na(Values) & !is.infinite(Values)])+by),by=by)
		H.counts <- matrix(ncol=(dim(Values)[2]),nrow=(length(breaks)-1))
		cat("calculating histograms \n")
		for(i in 1:dim(Values)[2]){
			dummy <- hist(Values[!is.na(Values[,i]) & !is.infinite(Values[,i]) & weights[,i]==1,i],breaks=breaks,plot=FALSE)
			if(i==1){
				H.mids <- dummy$mids
			}
			H.counts[,i] <- dummy$counts
		}
		if(draw.pairs){
			## set the color...
		}
		
		#with index you can specify what for channels will be drawn.
		if(!is.null(index)){
			#paint only the array(s) from the index, all on the same plot...
			#using the maximal values from all arrays
			xlim <- c(min(H.mids),max(H.mids))
			ylim <- c(0,max(H.counts))
			index <- unique(index)
			legendnames <- NULL
			if(!is.null(col)){
				if(length(col)<length(index)){
					col <- rep(col[1],length(index))
				}
			}
			else{
				col <- seq(1:length(index))
			}
			for(i in 1:length(index)){
				legendnames <- c(legendnames,names[index[i]])
				if(i==1){
					plot(H.mids,H.counts[,index[i]],col=col[i],xlim=xlim,ylim=ylim,type="l",xlab="Intensity (log2)",ylab="Counts",...)
				}
				else{
					points(H.mids,H.counts[,index[i]],col=col[i],type="l",...)
				}
			}
			if(draw.legend){
				legend(xlim[1],ylim[2],legendnames,col=col[1:length(index)],lty=1)
			}
		}
		else{
			#draw all histograms in one plot, use different colors...
			xlim <- c(min(H.mids),max(H.mids))
			ylim <- c(0,max(H.counts))
			if(!is.null(col)){
				if(length(col)<ncol(H.counts)){
					col <- rep(col[1],ncol(H.counts))
				}
			}
			else{
				col <- seq(1:ncol(H.counts))
			}
			#legendnames <- NULL
			for(i in 1: length(H.counts[1,])){
				if(i==1){
					plot(H.mids,H.counts[,i],col=col[i],xlim=xlim,ylim=ylim,type="l",xlab="Intensity (log2)",ylab="Counts",...)
				}
				else{
					points(H.mids,H.counts[,i],col=col[i],type="l",...)
				}
			}
			if(draw.legend){
				legend(xlim[1],ylim[2],names,col=col[1:ncol(H.counts)],lty=1)
			}
		}
	})


##################################################################################################
setMethod("drawHistogram","MadbSet",
	function(object,index=NULL,type=NULL,draw.pairs=FALSE,nr.breaks=250,draw.legend=FALSE,col=NULL,...){
		Values <- exprs(object)
		if(!.is.log(Values)){
			Values <- log2(Values)
			#cat("dim values",dim(Values),"\n")
		}
		
		names <- colnames(exprs(object))
		weights <- getWeights(object)
		by <- max(Values[!is.na(Values) & !is.infinite(Values)])/nr.breaks
		breaks <- seq(min(0,Values[!is.na(Values) & !is.infinite(Values)]),(max(Values[!is.na(Values) & !is.infinite(Values)])+by),by=by)
		H.counts <- matrix(ncol=(dim(Values)[2]),nrow=(length(breaks)-1))
		cat("calculating histograms \n")
		for(i in 1:dim(Values)[2]){
			dummy <- hist(Values[!is.na(Values[,i]) & !is.infinite(Values[,i]) & weights[,i]==1,i],breaks=breaks,plot=FALSE)
			if(i==1){
				H.mids <- dummy$mids
			}
			H.counts[,i] <- dummy$counts
		}
		if(draw.pairs){
			## set the color...
		}
		
		#with index you can specify what for channels will be drawn.
		if(!is.null(index)){
			#paint only the array(s) from the index, all on the same plot...
			#using the maximal values from all arrays
			xlim <- c(min(H.mids),max(H.mids))
			ylim <- c(0,max(H.counts))
			index <- unique(index)
			legendnames <- NULL
			if(!is.null(col)){
				if(length(col)<length(index)){
					col <- rep(col[1],length(index))
				}
			}
			else{
				col <- seq(1:length(index))
			}
			for(i in 1:length(index)){
				legendnames <- c(legendnames,names[index[i]])
				if(i==1){
					plot(H.mids,H.counts[,index[i]],col=col[i],xlim=xlim,ylim=ylim,type="l",xlab="Intensity (log2)",ylab="Counts",...)
				}
				else{
					points(H.mids,H.counts[,index[i]],col=col[i],type="l",...)
				}
			}
			if(draw.legend){
				legend(xlim[1],ylim[2],legendnames,col=col[1:length(index)],lty=1)
			}
		}
		else{
			#draw all histograms in one plot, use different colors...
			xlim <- c(min(H.mids),max(H.mids))
			ylim <- c(0,max(H.counts))
			if(!is.null(col)){
				if(length(col)<ncol(H.counts)){
					col <- rep(col[1],ncol(H.counts))
				}
			}
			else{
				col <- seq(1:ncol(H.counts))
			}
			#legendnames <- NULL
			for(i in 1: length(H.counts[1,])){
				if(i==1){
					plot(H.mids,H.counts[,i],col=col[i],xlim=xlim,ylim=ylim,type="l",xlab="Intensity (log2)",ylab="Counts",...)
				}
				else{
					points(H.mids,H.counts[,i],col=col[i],type="l",...)
				}
			}
			if(draw.legend){
				legend(xlim[1],ylim[2],names,col=col[1:ncol(H.counts)],lty=1)
			}
		}
	})




##################################################################################################
if( !isGeneric("drawBioControls") )
	setGeneric("drawBioControls", function(object,bio.5=c("AFFX-r2-Ec-bioB-5_at","AFFX-r2-Ec-bioC-5_at","AFFX-r2-Ec-bioD-5_at","AFFX-r2-P1-cre-5_at"),bio.3=c("AFFX-r2-Ec-bioB-3_at","AFFX-r2-Ec-bioC-3_at","AFFX-r2-Ec-bioD-3_at","AFFX-r2-P1-cre-3_at"),add=FALSE,col=NULL,log.scale=FALSE,main="",...)
	standardGeneric("drawBioControls"))

##################################################################################################
setMethod("drawBioControls","EexprSet",
	function(object,bio.5=c("AFFX-r2-Ec-bioB-5_at","AFFX-r2-Ec-bioC-5_at","AFFX-r2-Ec-bioD-5_at","AFFX-r2-P1-cre-5_at"),bio.3=c("AFFX-r2-Ec-bioB-3_at","AFFX-r2-Ec-bioC-3_at","AFFX-r2-Ec-bioD-3_at","AFFX-r2-P1-cre-3_at"),add=FALSE,col=NULL,log.scale=FALSE,main="",...){
	.Deprecated("","maDB",.MESSAGE_EEXPRSET_DEPRACATED)

	bio.conc <- c(1.5,5,25,100)
	nr.chips <- length(colnames(exprs(object)))
	dilution.5 <- matrix(ncol=nr.chips,nrow=4)
	dilution.3 <- matrix(ncol=nr.chips,nrow=4)

	if(.is.log(exprs(object))){
		for(i in 1:nr.chips){
			dilution.5[,i] <- 2^exprs(object)[bio.5,i]
			dilution.3[,i] <- 2^exprs(object)[bio.3,i]
		}
	}
	else{
		for(i in 1:nr.chips){
			dilution.5[,i] <- exprs(object)[bio.5,i]
			dilution.3[,i] <- exprs(object)[bio.3,i]
		}
	}
	if(log.scale){
		bio.conc <- log2(bio.conc)
		dilution.5 <- log2(dilution.5)
		dilution.3 <- log2(dilution.3)
	}
	
	ylim <- c(min(bio.conc),max(bio.conc))
	xlim <- c(min(dilution.3,dilution.5),max(dilution.3,dilution.5))
	if(is.null(col)){
		col = seq(1,nr.chips)
	}
	if(add){
		points(dilution.5[,1],bio.conc,col=col[1],type="b",...)
		points(dilution.3[,1],bio.conc,col=col[1],type="b",lty=2,...)
	}
	else{
		plot(dilution.5[,1],bio.conc,xlim=xlim,ylim=ylim,xlab="Intensity",ylab="Concentration",type="b",col=col[1],main=main,...)
		points(dilution.3[,1],bio.conc,col=col[1],type="b",lty=2,...)
	}
	for(i in 2:nr.chips){
		points(dilution.5[,i],bio.conc,col=col[i],type="b",...)
		points(dilution.3[,i],bio.conc,col=col[i],type="b",lty=2,...)
	}
	
	})
	

##################################################################################################
if( !isGeneric("drawPolyAControls") )
	setGeneric("drawPolyAControls", function(object,polyA.5=c("AFFX-r2-Bs-lys-5_at","AFFX-r2-Bs-phe-5_at","AFFX-r2-Bs-thr-5_s_at","AFFX-r2-Bs-dap-5_at"),polyA.3=c("AFFX-r2-Bs-lys-3_at","AFFX-r2-Bs-phe-3_at","AFFX-r2-Bs-thr-3_s_at","AFFX-r2-Bs-dap-3_at"),add=FALSE,col=NULL,log.scale=FALSE,main="",...)
	standardGeneric("drawPolyAControls"))

##################################################################################################
setMethod("drawPolyAControls","EexprSet",
	function(object,polyA.5=c("AFFX-r2-Bs-lys-5_at","AFFX-r2-Bs-phe-5_at","AFFX-r2-Bs-thr-5_s_at","AFFX-r2-Bs-dap-5_at"),polyA.3=c("AFFX-r2-Bs-lys-3_at","AFFX-r2-Bs-phe-3_at","AFFX-r2-Bs-thr-3_s_at","AFFX-r2-Bs-dap-3_at"),add=FALSE,col=NULL,log.scale=FALSE,main="",...){
	.Deprecated("","maDB",.MESSAGE_EEXPRSET_DEPRACATED)

	polyA.conc <- c(1/100000,1/50000,1/25000,1/7500)
	nr.chips <- length(colnames(exprs(object)))
	polyA.data.5 <- matrix(ncol=nr.chips,nrow=4)
	polyA.data.3 <- matrix(ncol=nr.chips,nrow=4)
	
	if(.is.log(exprs(object))){
		for(i in 1:nr.chips){
			polyA.data.5[,i] <- 2^exprs(object)[polyA.5,i]
			polyA.data.3[,i] <- 2^exprs(object)[polyA.3,i]
		}
	}
	else{
		for(i in 1:nr.chips){
			polyA.data.5[,i] <- exprs(object)[polyA.5,i]
			polyA.data.3[,i] <- exprs(object)[polyA.3,i]
		}
	}
	if(log.scale){
		polyA.conc <- log2(polyA.conc)
		polyA.data.5 <- log2(polyA.data.5)
		polyA.data.3 <- log2(polyA.data.3)
	}
	
	ylim = c(min(polyA.conc),max(polyA.conc))
	xlim = c(min(polyA.data.3,polyA.data.5),max(polyA.data.3,polyA.data.5))
	if(is.null(col)){
		col = seq(1,nr.chips)
	}
	if(add){
		points(polyA.data.5[,1],polyA.conc,col=col[1],type="b",...)
		points(polyA.data.3[,1],polyA.conc,col=col[1],type="b",lty=2,...)
	}
	else{
		plot(polyA.data.5[,1],polyA.conc,xlim=xlim,ylim=ylim,xlab="Intensity",ylab="Concentration",type="b",col=col[1],main=main,...)
		points(polyA.data.3[,1],polyA.conc,col=col[1],type="b",lty=2,...)
	}
	for(i in 2:nr.chips){
		points(polyA.data.5[,i],polyA.conc,col=col[i],type="b",...)
		points(polyA.data.3[,i],polyA.conc,col=col[i],type="b",lty=2,...)
	}
	
	})
	

##################################################################################################
.createResultTable <- function(object=NULL,affyIDs=NULL,chip=NULL,anncols=c("Probe","GenBank","UniGene","Description","Chromosome","Chromosome Location"),include.exprs=NULL){
	if(is.null(affyIDs)){
		stop("Please submit the affyIDs for which you want to create the table!\n\n")
	}
	if(is.null(chip)){
		stop("Please define the Affymetrix GeneChip or another identifier for your array that you have used (e.g. hgu133plus2). \n\n")
	}
	anntable <- aafTableAnn(affyIDs,chip,colnames=anncols)
#	if(!is.null(include.exprs) & !is.null(object)){
#		for(i in 1:length(include.exprs)){
#			dummy <- aafTable()
#		}
#	}
	anntable
}



##########################################################################
## 
##
setClass("Sample",representation(name="character",genus="character",species="character",line="character",individual="character",sample.provider="character",sex="character",age="character",developmental.state="character",tissue="character",celltype="character",genetic.modifications="character",disease.state="character",ref.to.clinical.data="character",growth.conditions="character",in.vivo.treatments="character",tissue.harvesting.method="character",extract.method="character",molecule.type="character",amplification="character",amount="character",treatment="character",exposure.time="character",description="character"),
	prototype=list(name="",genus="",species="",line="",individual="",sample.provider="",sex="",age="",developmental.state="",tissue="",celltype="",genetic.modifications="",disease.state="",ref.to.clinical.data="",growth.conditions="",in.vivo.treatments="",tissue.harvesting.method="",extract.method="",molecule.type="",amplification="",amount="",treatment="",exposure.time="",description="")
	)

#if( !isGeneric("createSampleDBTable") )
#	setGeneric("createSampleDBTable", function(Sample,Con)
#	standardGeneric("createSampleDBTable"))


##################################################################################################
if( !isGeneric("getContentMatrix") )
	setGeneric("getContentMatrix", function(object)
	standardGeneric("getContentMatrix"))


##################################################################################################
setMethod("getContentMatrix","Sample",
	function(object){
		slots <- slotNames(object)
		content <- matrix(ncol=length(slots),nrow=1)
		colnames(content) <- replaceAll(slots,".","_")
		for(i in 1:length(slots)){
			content[1,i] <- slot(object,slots[i])
		}
		content
	}
)





##
#	object that holds information about the hybridization and labeling
setClass("SignalChannel",representation(hyb.date="character",label.date="character",label.incorporation="character",color="character",description="character",array.index="numeric",sample.index="numeric",array.name="character"),
	prototype=list(label.date="",hyb.date="",color="",label.incorporation="",description="",array.index=0,sample.index=0,array.name="")
	)


##################################################################################################
setMethod("getContentMatrix","SignalChannel",
	function(object){
		slots <- slotNames(object)
		content <- matrix(ncol=length(slots),nrow=1)
		colnames(content) <- replaceAll(slots,".","_")
		for(i in 1:length(slots)){
			content[1,i] <- slot(object,slots[i])
		}
		content
	}
)

##################################################################################################
setMethod("show","SignalChannel",
	function(object){
		cat("Signal channel information: \n\n")
		cat(paste("hybridization date: ",object@hyb.date,"\n"))
		cat(paste("labeling date: ",object@label.date,"\n"))
		cat(paste("channel color : ",object@color,"\n"))
		cat(paste("description: ",object@description,"\n"))
		cat(paste("channel belongs to array: ",object@array.name,"\n\n"))
		cat(paste(""))
	}
)



##
# go the way back, create a new Sample from the data in the database.
createSampleFromTable <- function(table){
	if(length(table[,1])==1){
		sample <- new("Sample")
		slots <- replaceAll(slotNames(sample),".","_")
		CNames <- colnames(table)
		slots.idx <- multiGrep(CNames,slots,exact.match=TRUE)
		slots <- slots[slots.idx]
		for(j in 1:length(slots)){
			if(!is.null(table[1,slots[j]])){
				slot(sample,replaceAll(slots[j],"_",".")) <- table[1,slots[j]]
			}
		}
	}
	else{
		sample <- NULL
		slots <- replaceAll(slotNames(sample),".","_")
		CNames <- colnames(table)
		slots.idx <- multiGrep(CNames,slots,exact.match=TRUE)
		slots <- slots[slots.idx]
		for(i in 1:length(table[,1])){
			dummySample <- new("Sample")
			for(j in 1:length(slots)){
				if(!is.null(table[i,slots[j]])){
					slot(dummySample,replaceAll(slots[j],"_",".")) <- table[i,slots[j]]
				}
			}
			sample <- c(sample,list(dummySample))
		}
	}
	sample
}


#################
# draw a volcano plot
#
drawVolcanoPlot <- function(M,p,m=1,p.cut=0.05,p.transform=log10,ylab=NULL,colramp=NULL,na.rm=TRUE,...){
	Colors <- 1
	colored <- FALSE
	if(na.rm){
		M <- M[!is.na(M) & !is.na(p)]
		p <- p[!is.na(M) & !is.na(p)]
	}
	plog <- -p.transform(as.numeric(p))
	plog <- plog[!is.infinite(plog)]
	M <- M[!is.infinite(plog)]
	if(!require("geneplotter",character.only=TRUE)){
		colramp=NULL
		warning("library geneplotter which is not installed is required to draw a density colored MA plot!\n")
	}
	if(!is.null(colramp)){
		colored <- FALSE
		Colors <- tryCatch(densCols(M,plog,colramp=colramp),error=function(error){1})
	}
	if(colored){
		redvalue <- max(abs(plog),1)
		bluevalue <- max(abs(M),1)
		Colors <- rgb(r=abs(plog)/redvalue,g=abs(M)/bluevalue,b=0)
	}
	if(is.null(ylab)){
		ylab <- "-log10(p)"
	}
	plot(M,plog,col=Colors,xlab="Average M",ylab=ylab,...)
	abline(v=c(m,-m),col="grey",lty=2)
	abline(h=-log10(p.cut),col="grey",lty=2)
}


###################################################
##
##
if( !isGeneric("average") )
	setGeneric("average", function(object,average.which,log.scale=TRUE,method,array.names=NULL,v=TRUE,average.genes=TRUE,exclude.flagged=TRUE,only.good.spots=FALSE,...)
	standardGeneric("average"))

setMethod("average","EexprSet",
	function(object,average.which,log.scale=TRUE,method,array.names=NULL,v=TRUE,average.genes=TRUE,exclude.flagged=TRUE,only.good.spots=FALSE,filter=NULL,...){
		.Deprecated("drawHistogram","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
#		require(maDB)
		#require(tkfiDB)
		method <- match.arg(method,c("mean","median","tukey.biweight"))
		BigE <- exprs(object)
		####
		## new: filter.
		if(!is.null(filter)){
			if(filter!="range"){
				cat("Unknown filter:",filter,"discarding...\n")
				filter=NULL
			}
			else{
				if(!log.scale & !.is.log(BigE)){
					cat("The selected filter requires the data to be log2 transformed! transforming...\n")
					log.scale=TRUE
				}
			}
		}

		switch(method,
			mean = {themethod <- mean},
			median = {themethod <- median},
			tukey.biweight = {themethod <- tukey.biweight}
			)
		if(length(average.which)!=ncol(exprs(object))){
			stop("The parameter average.which must have the same length as the exprs slot of the submitted EexprSet class has columns!\n")
		}

		if(log.scale & !.is.log(BigE)){
			BigE <- log2(BigE)
		}
		weights <- getWeights(object)
		if(!exclude.flagged){
			weights[,] <- 1
		}
		else{
			for(i in 1:ncol(weights)){
				BigE[weights[,i]==0,i] <- NA
			}
		}
		
		##
		## combine columns of the BigE (replicated Arrays)
		if(!is.null(filter)){
			if(filter=="range"){
				tempMatrix <- as.matrix(apply(BigE,MARGIN=1,.averageRowsRangeFilter,method=themethod,which=average.which,accept.no.na=only.good.spots,...))
			}
		}
		else{
			tempMatrix <- as.matrix(apply(BigE,MARGIN=1,.averageRows,method=themethod,which=average.which,accept.no.na=only.good.spots))
		}
		if(ncol(tempMatrix)==nrow(BigE)){
			SmallerE <- t(tempMatrix)
		}
		else{
			SmallerE <- tempMatrix
		}
		rm(tempMatrix)
		#cat("nrow, ncol of SmallerE: ",nrow(SmallerE),ncol(SmallerE))
		#cat(ncol(SmallerE),nrow(SmallerE),"\n")
		if(is.null(array.names)){
			array.names <- colnames(BigE)[unique(average.which)]
		}
		colnames(SmallerE) <- array.names[1:ncol(SmallerE)]
		rownames(SmallerE) <- rownames(exprs(object))
		if(v){
			cat(ncol(BigE)," columns where combined to ",ncol(SmallerE)," columns\n")
		}
		##
		# combine rows of BigE (if IDs are not unique)
		ids <- rownames(SmallerE)
		if(length(ids)!=length(unique(ids)) & average.genes){
			if(v){
				cat("averaging replicated genes per array\n")
			}
			ids <- unique(ids)
			ids <- ids[!is.na(ids)]
			SmallE <- matrix(ncol=ncol(SmallerE),nrow=length(ids))
			if(nrow(object@genes)==nrow(exprs(object))){
				genes <- matrix(ncol=ncol(object@genes),nrow=length(ids))
				colnames(genes) <- colnames(object@genes)
				rownames(genes) <- ids
			}
			else{
				genes <- NULL
			}
			colnames(SmallE) <- colnames(SmallerE)
			rownames(SmallE) <- ids
			for(i in 1:length(ids)){
				bm <- rownames(SmallerE)==ids[i]
				bm[is.na(bm)] <- FALSE
				for(j in 1:ncol(SmallE)){
					SmallE[i,j] <- themethod(SmallerE[bm,j],na.rm=TRUE)
				}
				if(!is.null(genes)){
					tempp <- object@genes[bm,]
					if(!is.null(nrow(tempp))){
						tempp <- tempp[1,]
					}
					genes[i,] <- tempp
				}
			}
			SmallerE <- SmallE
			if(v){
				cat(nrow(BigE)," rows where combined to ",length(ids)," rows\n")
			}
			if(!is.null(genes)){
				object@genes <- genes
			}
		}
		SmallerE <- as.matrix(SmallerE)
		if(object@type=="TwoColor"){
			# have to define array pairs...
			array.pairs <- NULL
			for(i in 1:(ncol(SmallerE)/2)){
				array.pairs <- c(array.pairs,i,-i)
			}
			object@array.pairs <- array.pairs
		}
		
		##
		# process the weights...
		weights <- matrix(ncol=ncol(SmallerE),nrow=nrow(SmallerE))
		weights[,] <- 1
		colnames(weights) <- colnames(SmallerE)
		rownames(weights) <- rownames(SmallerE)
		for(i in 1:ncol(SmallerE)){
			weights[is.na(SmallerE[,i]),i] <- 0
		}
		object@weights <- weights
		exprs(object) <- SmallerE
		object
	}
)


setMethod("average","MadbSet",
	function(object,average.which,log.scale=TRUE,method,array.names=NULL,v=TRUE,average.genes=TRUE,exclude.flagged=TRUE,only.good.spots=FALSE,filter=NULL,...){
#		require(maDB)
		#require(tkfiDB)
		method <- match.arg(method,c("mean","median","tukey.biweight"))
		BigE <- exprs(object)
		####
		## new: filter.
		if(!is.null(filter)){
			if(filter!="range"){
				cat("Unknown filter:",filter,"discarding...\n")
				filter=NULL
			}
			else{
				if(!log.scale & !.is.log(BigE)){
					cat("The selected filter requires the data to be log2 transformed! transforming...\n")
					log.scale=TRUE
				}
			}
		}

		switch(method,
			mean = {themethod <- mean},
			median = {themethod <- median},
			tukey.biweight = {themethod <- tukey.biweight}
			)
		if(length(average.which)!=ncol(exprs(object))){
			stop("The parameter average.which must have the same length as the exprs slot of the submitted EexprSet class has columns!\n")
		}

		if(log.scale & !.is.log(BigE)){
			BigE <- log2(BigE)
		}
		weights <- getWeights(object)
		if(!exclude.flagged){
			weights[,] <- 1
		}
		else{
			for(i in 1:ncol(weights)){
				BigE[weights[,i]==0,i] <- NA
			}
		}
		
		##
		## combine columns of the BigE (replicated Arrays)
		if(!is.null(filter)){
			if(filter=="range"){
				tempMatrix <- as.matrix(apply(BigE,MARGIN=1,.averageRowsRangeFilter,method=themethod,which=average.which,accept.no.na=only.good.spots,...))
			}
		}
		else{
			tempMatrix <- as.matrix(apply(BigE,MARGIN=1,.averageRows,method=themethod,which=average.which,accept.no.na=only.good.spots))
		}
		if(ncol(tempMatrix)==nrow(BigE)){
			SmallerE <- t(tempMatrix)
		}
		else{
			SmallerE <- tempMatrix
		}
		rm(tempMatrix)
		#cat("nrow, ncol of SmallerE: ",nrow(SmallerE),ncol(SmallerE))
		#cat(ncol(SmallerE),nrow(SmallerE),"\n")
		if(is.null(array.names)){
			array.names <- colnames(BigE)[unique(average.which)]
		}
		colnames(SmallerE) <- array.names[1:ncol(SmallerE)]
		rownames(SmallerE) <- rownames(exprs(object))
		if(v){
			cat(ncol(BigE)," columns where combined to ",ncol(SmallerE)," columns\n")
		}
		##
		# combine rows of BigE (if IDs are not unique)
		## trying to re-add the possibility to average replicated spots!
		IDcol <- tolower(colnames(object@genes)) == "id"
		if(any(IDcol) & average.genes){
			ids <- object@genes[, IDcol]
			if( sum(rownames(SmallerE) == ids )!= length(ids)){
				rownames(SmallerE) <- as.character(ids)
			}
		}
		ids <- rownames(SmallerE)
		if(length(ids)!=length(unique(ids)) & average.genes){
			if(v){
				cat("averaging replicated genes per array\n")
			}
			ids <- unique(ids)
			ids <- ids[!is.na(ids)]
			SmallE <- matrix(ncol=ncol(SmallerE),nrow=length(ids))
			if(nrow(object@genes)==nrow(exprs(object))){
				genes <- matrix(ncol=ncol(object@genes),nrow=length(ids))
				colnames(genes) <- colnames(object@genes)
				rownames(genes) <- ids
			}
			else{
				genes <- NULL
			}
			colnames(SmallE) <- colnames(SmallerE)
			rownames(SmallE) <- ids
			for(i in 1:length(ids)){
				bm <- rownames(SmallerE)==ids[i]
				bm[is.na(bm)] <- FALSE
				for(j in 1:ncol(SmallE)){
					SmallE[i,j] <- themethod(SmallerE[bm,j],na.rm=TRUE)
				}
				if(!is.null(genes)){
					tempp <- object@genes[bm,]
					if(!is.null(nrow(tempp))){
						tempp <- tempp[1,]
					}
					genes[i,] <- tempp
				}
			}
			SmallerE <- SmallE
			if(v){
				cat(nrow(BigE)," rows where combined to ",length(ids)," rows\n")
			}
			if(!is.null(genes)){
				object@genes <- genes
			}
		}
		SmallerE <- as.matrix(SmallerE)
		if(object@type=="TwoColor"){
			# have to define array pairs...
			array.pairs <- NULL
			for(i in 1:(ncol(SmallerE)/2)){
				array.pairs <- c(array.pairs,i,-i)
			}
			object@array.pairs <- array.pairs
		}
		
		##
		# process the weights...
		weights <- matrix(ncol=ncol(SmallerE),nrow=nrow(SmallerE))
		weights[,] <- 1
		colnames(weights) <- colnames(SmallerE)
		rownames(weights) <- rownames(SmallerE)
		for(i in 1:ncol(SmallerE)){
			weights[is.na(SmallerE[,i]),i] <- 0
		}
		object@weights <- weights
		exprs(object) <- SmallerE
        ## have to do something with the phenoData... in the mean time just copy...
        dummy <- data.frame(sample=1:ncol(SmallerE))
        rownames(dummy) <- colnames(SmallerE)
        otherdummy <- data.frame(labelDescription=c("Arbitrary numbering"))
        rownames(otherdummy) <- c("sample")
        phenoData(object) <- new("AnnotatedDataFrame",data=dummy,varMetadata=otherdummy)
		object
	}
)



.averageRows <- function(x,method=mean,which,accept.no.na=FALSE){
	wh <- sort(unique(which))
	res <- NULL
	#cat(wh,"\n")
	for(i in 1:length(wh)){
		#cat(x[which==wh[i]],"\n")
		if(accept.no.na & sum(is.na(x[which==wh[i]]))>0){
			res <- c(res,NA)
		}
		else{
			res <- c(res,method(x[which==wh[i]],na.rm=TRUE))
		}
	}
	res
}
##
###################################

####
## TODO: change this for experiments where more than 2 arrays should be averaged...
.averageRowsRangeFilter <- function(x,method=mean,which,accept.no.na=TRUE,pairwise=FALSE,cut=1){
	wh <- sort(unique(which))
	## check if we can perform a pairwise test:
	pair.test <- length(wh)/2
	if(pairwise){
		## requirements for pairwise:
		## 1) two color arrays, this means that the resulting number of signalchannels must! be even.
		## 2) each array must be replicated twice (this means tow arrays have to be averaged to one).
		pairwise=floor(pair.test)==length(wh)/2
		if(pairwise){
			pairwise=length(which)==2*length(wh)
		}
		if(!pairwise){
			cat("WARNING: pairwise was set to TRUE, but the number of signal channels is not even. can not perform a range filter based on M values, will switch back to a range filter on expression values.\n")
			warning("pairwise was set to TRUE, but the number of signal channels is not even. can not perform a range filter based on M values, will switch back to a range filter on expression values.\n")
		}
	}
	##
	## do not accept any
	accept.no.na=TRUE
	res <- NULL
	#cat(wh,"\n")
	if(pairwise){
		for(i in seq(1,length(wh),by=2)){
			rs <- x[which==wh[i]]
			gs <- x[which==wh[i+1]]
			if(accept.no.na & sum(is.na(x[which==wh[i]]))>0){
				res <- c(res,NA,NA)
			}
			else{
				if( abs(diff( c( (rs[1]-gs[1]), (rs[2]-gs[2]) ) ))>cut ){
					## means the difference between the M values is bigger than the cut level.
					res <- c(res,NA,NA)
				}
				else{
					##ok, calculate...
					res <- c(res,method(x[which==wh[i]],na.rm=TRUE))
					res <- c(res,method(x[which==wh[i+1]],na.rm=TRUE))
				}
			}
		}
	}
	else{
		for(i in 1:length(wh)){
			#cat(x[which==wh[i]],"\n")
			if(accept.no.na & sum(is.na(x[which==wh[i]]))>0){
				res <- c(res,NA)
			}
			else{
				## flag features, if the range of their values exceeds the specified cut off.
				if(abs(diff(range(x[which==wh[i]]))) > cut){
					res <- c(res,NA)
				}
				else{
					res <- c(res,method(x[which==wh[i]],na.rm=TRUE))
				}
			}
		}
	}
	res
}













