## last checked: 03.05.2007
## this function in here require the pgUtils and/or the Rdbi packages!



if( !isGeneric("createNewDBTable") )
	setGeneric("createNewDBTable", function(object,Con,table.name=NULL,no.transaction=FALSE)
	standardGeneric("createNewDBTable"))

setMethod("createNewDBTable","Sample",
	function(object,Con,table.name=NULL,no.transaction=FALSE){
		require(pgUtils)
		if(is.null(table.name)){
			table.name <- "samples"
		}
		colname <- replaceAll(slotNames(object),".","_")
		createDBTable(Con,name=table.name,attributes=colname,no.transaction=no.transaction,no.clobber=FALSE)
	}
)

setMethod("createNewDBTable","SignalChannel",
	function(object,Con,table.name=NULL,no.transaction=FALSE){
		require(pgUtils)
		if(is.null(table.name)){
			table.name <- "signal_channels"
		}
		name <- replaceAll(slotNames(object),".","_")
		name <- name[name!="array_index"]
		name <- name[name!="sample_index"]
		#name <- name[name!="array_name"]
		createDBTable(Con,name=table.name,attributes=name,references=c("arrays","samples"),no.transaction=no.transaction)
	}
)

##
# definition: this function should return the primary key of a object in the database. if the object does already exist,
# the pk is returned, otherwise the object will be inserted first.
if( !isGeneric("getPK") )
	setGeneric("getPK", function(object,Con,table.name=NULL,all.fields=FALSE,...)
	standardGeneric("getPK"))


setMethod("getPK","Sample",
	function(object,Con,table.name=NULL,all.fields=FALSE,...){
		require(pgUtils)
		content <- getContentMatrix(object)
		if(is.null(table.name)){
			table.name <- "samples"
		}
		## reduce the matrix to the not empty fields...
		reduced.colnames <- colnames(content)[content!="" & !is.na(content)]
		reduced.content <- content[content!="" & !is.na(content)]
		where.string <- "WHERE"
		for(i in 1:length(reduced.content)){
			where.string <- paste(where.string,reduced.colnames[i],"=",rpgsql.format.values(reduced.content[i]))
			if(i!=length(reduced.content)){
				where.string <- paste(where.string,"AND")
			}
		}
		pk <- paste(table.name,"_pk",sep="")
		res <- dbSendQuery(Con,"SELECT",pk,"FROM",table.name,where.string)
		if(dbResultInfo(res)$error.message!=""){
			stop(paste("An error occurred: ",dbResultInfo(res)$error.message))
		}
		result <- dbGetResult(res)
		if(nrow(result)==0){
			cat("Inserting the sample into the database\n")
			insertIntoTable(Con,name=table.name,data=reduced.content,attributes=reduced.colnames,...)
			res <- dbSendQuery(Con,"SELECT",pk,"FROM",table.name,where.string)
			result <- dbGetResult(res)
		}
		pk <- result[1,1]
		pk
	}
)

if( !isGeneric("loadFromDB") )
	setGeneric("loadFromDB", function(object=NULL,connection=NULL,name=NULL,pk=NULL,v=TRUE,...)
	standardGeneric("loadFromDB"))

## make it possible to load a subset of genes!!! pk, id,...
setMethod("loadFromDB","EexprSet",
	function(object=NULL,connection=NULL,name=NULL,pk=NULL,v=TRUE,...){
		.Deprecated("loadFromDB","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
		require(pgUtils)
		if(is.null(connection)){
			stop("Please submit the connection to the database (created with the dbConnect function)\n\n")
		}
		if(is.null(name)){
			stop("Please submit the name by which the data can be identifyed. In this case the experiment title.\n To get the names from all  experiments in the database call dbGetExperimentInfo(connection).\n\n")
		}
		experiments <- dbGetExperimentInfo(connection,name)
		if(!is.null(pk)){
			bm <- rep(FALSE,length(experiments[,1]))
			for(i in 1:length(pk)){
				bm <- bm | experiments[,"signal_channels_pk"]==pk[i]
			}
			experiments <- experiments[bm,]
			if(v){
				cat(paste("Retrieving a subset of",length(experiments[,1]),"chips from the experiment",name,"\n"))
			}
		}
		first <- dbGetResult(dbSendQuery(connection,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[1,"signal_channels_pk"])," ORDER BY exp_values_pk"))
		expr.matrix <- matrix(nrow=nrow(first),ncol=length(experiments[,1]))
		rownames(expr.matrix) <- first[,"id"]
		expr.matrix[,1] <- first[,"value"]
		annotation(object) <- experiments[1,"chip"]
		
		##
		# retrieving the col names...
		if(length(unique(experiments[,"arrays_pk"]))==length(unique(experiments[,"signal_channels_pk"]))){
			if(v){
				cat("got one channel micro arrays...\n")
			}
			colnames(expr.matrix) <- experiments[,"array_name"]
			object@type <- "OneColor"
		}
		else{
			##
			# have to load all the signal channels and check what for colors are used...
			object@type <- "TwoColor"
			if(v){
				cat("got two color micro arrays...\n")
			}
			col.names <- NULL
			for(i in 1:length(experiments[,1])){
				dummy.result <- dbGetResult(dbSendQuery(con,"SELECT * FROM signal_channels WHERE signal_channels_pk=",experiments[i,"signal_channels_pk"]))
				dummy.color <- " Red"
				if(dummy.result[1,"color"]=="Cy5"){
					dummy.color <- " Red"
				}
				else{
					dummy.color <- " Green"
				}
				col.names <- c(col.names,paste(dummy.result[1,"array_name"],dummy.color,sep=""))
			}
			colnames(expr.matrix) <- col.names
			#stop("Sorry, two color micro arrays are not implemented yet... coming soon...\n\n")
		}
		#cat(paste("loading the expression values from the database. \n Please wait for",length(experiments[,1]),"dots:"))
		#cat(".")
		if(v){
			cat(paste("loading the expression values from the database. \n"))
			progress(0,steps=40)
			p.counter <- 0
			l.e <- length(experiments[,1])
		}
		for(i in 2:length(experiments[,1])){
			res.values <- dbGetResult(dbSendQuery(connection,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[i,"signal_channels_pk"])," ORDER BY exp_values_pk"))
			## check if the ids are unique
			if(length(res.values[,"id"])==length(unique(res.values[,"id"]))){
				expr.matrix[res.values[,"id"],i] <- res.values[,"value"]
			}
			else{
				##
				# ids are not unique!!!
				#cat("ids not unique...\n")
				expr.matrix[,i] <- res.values[,"value"]
			}
			#cat(".")
			if(v){
				p.counter <- p.counter + 1
				progress(p.counter*100/l.e,steps=40)
			}
		}
		if(v){
			progress(100,steps=40)
			cat("\nfinished\n")
		}
		samples.pk <- (experiments[,"samples_fk"])
		samples <-  list()
		if(v){
			cat("retrieving the samples\n")
			progress(0,steps=40)
			p.counter <- 0
			l.s <- length(samples.pk)
		}
			
		for(i in 1:length(samples.pk)){
			samples <- c(samples,list(loadFromDB(new("Sample"),connection=connection,pk=samples.pk[i])))
			if(v){
				p.counter <- p.counter+1
				progress(p.counter*100/l.s,steps=40)
			}
		}
		object@samples <- samples
		if(v){
			progress(100,steps=40)
			cat("\nfinished\n")
		}
		exprs(object) <- expr.matrix
		object
	}
)


## make it possible to load a subset of genes!!! pk, id,...
setMethod("loadFromDB","MadbSet",
	function(object=NULL,connection=NULL,name=NULL,pk=NULL,v=TRUE,...){
		require(pgUtils)
		if(is.null(connection)){
			stop("Please submit the connection to the database (created with the dbConnect function)\n\n")
		}
		if(is.null(name)){
			stop("Please submit the name by which the data can be identifyed. In this case the experiment title.\n To get the names from all  experiments in the database call dbGetExperimentInfo(connection).\n\n")
		}
		experiments <- dbGetExperimentInfo(connection,name)
		if(!is.null(pk)){
			bm <- rep(FALSE,length(experiments[,1]))
			for(i in 1:length(pk)){
				bm <- bm | experiments[,"signal_channels_pk"]==pk[i]
			}
			experiments <- experiments[bm,]
			if(v){
				cat(paste("Retrieving a subset of",length(experiments[,1]),"chips from the experiment",name,"\n"))
			}
		}
		first <- dbGetResult(dbSendQuery(connection,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[1,"signal_channels_pk"])," ORDER BY exp_values_pk"))
		expr.matrix <- matrix(nrow=nrow(first),ncol=length(experiments[,1]))
		rownames(expr.matrix) <- first[,"id"]
		expr.matrix[,1] <- first[,"value"]
		annotation(object) <- experiments[1,"chip"]
		
		##
		# retrieving the col names...
		if(length(unique(experiments[,"arrays_pk"]))==length(unique(experiments[,"signal_channels_pk"]))){
			if(v){
				cat("got one channel micro arrays...\n")
			}
			colnames(expr.matrix) <- experiments[,"array_name"]
			object@type <- "OneColor"
		}
		else{
			##
			# have to load all the signal channels and check what for colors are used...
			object@type <- "TwoColor"
			if(v){
				cat("got two color micro arrays...\n")
			}
			col.names <- NULL
			for(i in 1:length(experiments[,1])){
				dummy.result <- dbGetResult(dbSendQuery(con,"SELECT * FROM signal_channels WHERE signal_channels_pk=",experiments[i,"signal_channels_pk"]))
				dummy.color <- " Red"
				if(dummy.result[1,"color"]=="Cy5"){
					dummy.color <- " Red"
				}
				else{
					dummy.color <- " Green"
				}
				col.names <- c(col.names,paste(dummy.result[1,"array_name"],dummy.color,sep=""))
			}
			colnames(expr.matrix) <- col.names
			#stop("Sorry, two color micro arrays are not implemented yet... coming soon...\n\n")
		}
		#cat(paste("loading the expression values from the database. \n Please wait for",length(experiments[,1]),"dots:"))
		#cat(".")
		if(v){
			cat(paste("loading the expression values from the database. \n"))
			progress(0,steps=40)
			p.counter <- 0
			l.e <- length(experiments[,1])
		}
		for(i in 2:length(experiments[,1])){
			res.values <- dbGetResult(dbSendQuery(connection,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[i,"signal_channels_pk"])," ORDER BY exp_values_pk"))
			## check if the ids are unique
			if(length(res.values[,"id"])==length(unique(res.values[,"id"]))){
				expr.matrix[res.values[,"id"],i] <- res.values[,"value"]
			}
			else{
				##
				# ids are not unique!!!
				#cat("ids not unique...\n")
				expr.matrix[,i] <- res.values[,"value"]
			}
			#cat(".")
			if(v){
				p.counter <- p.counter + 1
				progress(p.counter*100/l.e,steps=40)
			}
		}
		if(v){
			progress(100,steps=40)
			cat("\nfinished\n")
		}
		samples.pk <- (experiments[,"samples_fk"])
		samples <-  list()
		if(v){
			cat("retrieving the samples\n")
			progress(0,steps=40)
			p.counter <- 0
			l.s <- length(samples.pk)
		}
			
		for(i in 1:length(samples.pk)){
			samples <- c(samples,list(loadFromDB(new("Sample"),connection=connection,pk=samples.pk[i])))
			if(v){
				p.counter <- p.counter+1
				progress(p.counter*100/l.s,steps=40)
			}
		}
		object@samples <- samples
		if(v){
			progress(100,steps=40)
			cat("\nfinished\n")
		}
		exprs(object) <- expr.matrix
		D <- data.frame(sample=1:ncol(expr.matrix))
		rownames(D) <- colnames(expr.matrix)
		V <- data.frame(labelDescription="sample numbering")
		rownames(V) <- "sample"
		phenoData(object) <- new("AnnotatedDataFrame",data=D,varMetadata=V)
		object
	}
)



setMethod("loadFromDB","Sample",
	function(object=NULL,connection=NULL,name=NULL,pk=NULL,v=TRUE,...){
		require(pgUtils)
		if(is.null(pk)){
			stop("Please submit the primary key of your sample! If you want to know what samples are in the database, simply use the getSamplesFromDB call.\n\n")
		}
		sampl <- dbGetResult(dbSendQuery(connection,"SELECT * FROM samples WHERE samples_pk=",rpgsql.format.values(pk)))
		object <- createSampleFromTable(sampl)
		object
	}
)

if( !isGeneric("publishToDB") )
	setGeneric("publishToDB", function(object=NULL,connection=NULL,exp.name=NULL,exp.date=NULL,samples=NULL,signal.channels=NULL,preprocessing=NULL,pubmed.id=NULL,global.transaction=TRUE,v=TRUE)
	standardGeneric("publishToDB"))

	
##
#	what does this function: writing a EexprSet into a database. a unique experiment name has to be submitted, the arrays from this experiment are
#	added, then the signal channels, 
#
setMethod("publishToDB","EexprSet",
	function(object=NULL,connection=NULL,exp.name=NULL,exp.date=NULL,samples=NULL,signal.channels=NULL,preprocessing=NULL,pubmed.id=NULL,global.transaction=TRUE,v=TRUE){
		.Deprecated("publishToDB","maDB",.MESSAGE_EEXPRSET_DEPRACATED)
		require(pgUtils)
		if(is.null(object)){
			stop("please submit a array experiment\n\n")
		}
		if(is.null(connection)){
			stop("please open first a connection to your database.\n")
		}
		if(is.null(exp.date)){
			exp.date="18011977"
		}
		Con <- connection
		if(is.null(exp.name)){
			if(!is.null(description(object)@title)){
				exp.name <- description(object)@title
			}
			else{
				stop("please specify the experiment name!\n\n")
			}
		}
		
		##
		# if an error occurs, rollback the transaction!
		#on.exit(.resetAfterError())
		# have to be shure that a connection object exists also after an error...
		#assign("ConnectionForErrors",Con,env=globalenv())
		#options(error=expression(.rollItBack()))
		
		
		#if(is.null(samples)){
		#	stop("please specify the samples that are on the chips\n\n")
		#}
		#library(tkfiDB)
		#Con <- dbConnect(PgSQL(),dbname=dbname,host=host,user=user)
		#show(Con)
		
		if(global.transaction){
			r <- dbSendQuery(Con,"BEGIN TRANSACTION;")
		}
		
		tryCatch(.publishExperimentToDB(object,Con,exp.name,exp.date,samples,signal.channels,preprocessing=preprocessing,pubmed.id=pubmed.id,global.transaction,v),error=function(error){
			.rollItBack(Con)
			cat("\n",as.character(error),"\n","To get more information read the log file:",log.file,"\n")
		},interrupt=function(interrupt){
			.rollItBack(Con)
			cat("The transaction has been rolled back.\n")
		})
		

		#if(global.transaction){
		#	on.exit(dbSendQuery(Con, "ROLLBACK TRANSACTION"))
		#}
	}
)


##
#	what does this function: writing a EexprSet into a database. a unique experiment name has to be submitted, the arrays from this experiment are
#	added, then the signal channels, 
#
setMethod("publishToDB","MadbSet",
	function(object=NULL,connection=NULL,exp.name=NULL,exp.date=NULL,samples=NULL,signal.channels=NULL,preprocessing=NULL,pubmed.id=NULL,global.transaction=TRUE,v=TRUE){
		require(pgUtils)
		if(is.null(object)){
			stop("please submit a array experiment\n\n")
		}
		if(is.null(connection)){
			stop("please open first a connection to your database.\n")
		}
		if(is.null(exp.date)){
			exp.date="18011977"
		}
		Con <- connection
		if(is.null(exp.name)){
			if(!is.null(description(object)@title)){
				exp.name <- description(object)@title
			}
			else{
				stop("please specify the experiment name!\n\n")
			}
		}
		
		##
		# if an error occurs, rollback the transaction!
		#on.exit(.resetAfterError())
		# have to be shure that a connection object exists also after an error...
		#assign("ConnectionForErrors",Con,env=globalenv())
		#options(error=expression(.rollItBack()))
		
		
		#if(is.null(samples)){
		#	stop("please specify the samples that are on the chips\n\n")
		#}
		#library(tkfiDB)
		#Con <- dbConnect(PgSQL(),dbname=dbname,host=host,user=user)
		#show(Con)
		
		if(global.transaction){
			r <- dbSendQuery(Con,"BEGIN TRANSACTION;")
		}
		
		tryCatch(.publishExperimentToDB(object,Con,exp.name,exp.date,samples,signal.channels,preprocessing=preprocessing,pubmed.id=pubmed.id,global.transaction,v),error=function(error){
			.rollItBack(Con)
			cat("\n",as.character(error),"\n","To get more information read the log file:",log.file,"\n")
		},interrupt=function(interrupt){
			.rollItBack(Con)
			cat("The transaction has been rolled back.\n")
		})
		

		#if(global.transaction){
		#	on.exit(dbSendQuery(Con, "ROLLBACK TRANSACTION"))
		#}
	}
)



.publishExperimentToDB <- function(object=NULL,Con=NULL,exp.name=NULL,exp.date=NULL,samples=NULL,signal.channels=NULL,preprocessing="",pubmed.id="",global.transaction=TRUE,v=TRUE){
		
	chips <- colnames(exprs(object))
	table.list <- dbListTables(Con)
	if(is.null(preprocessing)){
		preprocessing=""
	}
	if(is.null(pubmed.id)){
		pubmed.id=""
	}

	if( !("experiments" %in% table.list$relname)){
		if(v){
			cat("creating new experiments table...\n")
		}
		createDBTable(Con,name="experiments",attributes=c("title","description","exp_date","preprocessing","pubmed_id","publish_date"),options=c("UNIQUE","","","","",""),no.transaction=global.transaction,silent=TRUE)
	}
	if( !("arrays" %in% table.list$relname)){
		if(v){
			cat("creating new arrays table...\n")
		}
		createDBTable(Con,name="arrays",attributes=c("arrayname","type","chip"),references="experiments",no.transaction=global.transaction,silent=TRUE)
	}
	
	if( !("samples" %in% table.list$relname)){
		if(v){
			cat("creating new samples table...\n")
		}
		#colname <- replaceAll(slotNames(new("Sample")),old=".",new="_")
		#createDBTable(Con,name="samples",attributes=colname,no.transaction=global.transaction,silent=TRUE)
		createNewDBTable(new("Sample"),Con,no.transaction=global.transaction)
	}

	
	if(!("signal_channels" %in% table.list$relname)){
		if(v){
			cat("creating new signal_channels table...\n")
		}
		createNewDBTable(new("SignalChannel"),Con,no.transaction=global.transaction)
	}
	
			
	if( !("exp_values" %in% table.list$relname)){
		if(v){
			cat("creating new values table...\n")
		}
		createDBTable(Con,name="exp_values",attributes=c("id","value","log_value"),data.types=c("TEXT","float8","BOOL"),references="signal_channels",no.transaction=global.transaction,silent=TRUE)
	}

	if(v){
		cat("finished creating tables...\n")
	}
	if(global.transaction){
		r <- dbSendQuery(Con,"COMMIT TRANSACTION;")
		r <- dbSendQuery(Con,"BEGIN TRANSACTION;")
	}
#		if(global.transaction){
#			dbSendQuery(Con,"COMMIT TRANSACTION;")
#			dbSendQuery(Con,"BEGIN TRANSACTION;")
#			cat("committed...\n")
#		}
			
	if(!is.null(samples)){
		# let's start...
#		sample.pks <- matrix(ncol=length(samples),nrow=1)
		sample.pks <- character(length(samples))
		for(i in 1:length(samples)){
			sample.pks[i] <- getPK(samples[[i]],Con,no.transaction=global.transaction)
		}
	}
	else{
#		sample.pks <- matrix(ncol=1,nrow=1)
		sample.pks <- getPK(new("Sample",name="Unknown"),Con,no.transaction=global.transaction)
	}

	if(is.null(signal.channels)){
		signal.channels <- getSignalChannels(object)
		if(is.null(samples)){
			for(i in 1:length(signal.channels)){
				signal.channels[[i]]@sample.index <- 1
			}
		}
	}
		
	array.names <- getArrays(object)
	##
	# have to check if the SignalChannels match to the samples...
	for(i in 1:length(signal.channels)){
		if(signal.channels[[i]]@sample.index>length(sample.pks)){
			dbSendQuery(Con,"ROLLBACK TRANSACTION")
			stop("The sample.index attribute in the submitted SignalChannel exceeds the number of samples submitted!!\n")
		}
		if(signal.channels[[i]]@array.index>length(array.names)){
			dbSendQuery(Con,"ROLLBACK TRANSACTION")
			stop("The array.index attribute in the submitted SignalChannel exceeds the number of arrays submitted!!\n")
		}
	}
		
	insertIntoTable(Con,name="experiments",data=c(exp.name,exp.date,date(),description(object)@abstract,preprocessing,pubmed.id),attributes=c("title","exp_date","publish_date","description","preprocessing","pubmed_id"),no.transaction=global.transaction)
	cat("Experiment added to the experiments table...\n")
	
	#for(i in 1:length(samples)){
	#	current <- samples[[i]]
	#	data <- getContentMatrix(current)
	#	insertIntoTable(Con,name="samples",data=c(data[1,],exp.name),attributes=c(colnames(data),"exp_name"))
	#	
	#}
	##
	# inserting arrays:
	if(v){
		cat("Adding the arrays to the arrays_table\n")
		l.a <- length(array.names)
		p.counter <- 0
		progress(0,steps=40)
	}
	for(i in 1:length(array.names)){
		#cat("array name",array.names[i],"\n")
		insertIntoTable(Con,name="arrays",data=c(array.names[i],object@type,annotation(object),exp.name),attributes=c("arrayname","type","chip","exp_name"),references=new("AutoReference",source.table="arrays",ref.table="experiments",source.table.column="exp_name",ref.table.column="title"),no.transaction=global.transaction)
		if(v){
			p.counter <- p.counter + 1
			progress(p.counter*100/l.a,steps=40)
		}
	}
	if(v){
		progress(100,steps=40)
		cat("\nfinished\n")
		cat("Adding the signal channels to the signal_channels table\n")
		l.c <- length(chips)
		progress(0,steps=40)
		p.counter <- 0
	}

	sig.channel.pks <- matrix(ncol=length(chips),nrow=1)
	array.ref <- new("AutoReference",source.table="signal_channels",ref.table="arrays",source.table.column="array_index",ref.table.column="arrays_pk")
	sample.ref <- new("AutoReference",source.table="signal_channels",ref.table="samples",source.table.column="sample_index",ref.table.column="samples_pk")
	for(i in 1:length(chips)){
		subquery <- paste("( SELECT experiments_pk FROM experiments WHERE title=",rpgsql.format.values(exp.name),")")
		res <- dbSendQuery(Con,"SELECT arrays_pk FROM arrays WHERE arrayname=",rpgsql.format.values(array.names[signal.channels[[i]]@array.index]),"AND experiments_fk=",subquery)
			
		#array.pks[i] <- as.character(dbGetResult(res)[[1]])
		##
		# inserting the signal channel...
		sig.channel <- getContentMatrix(signal.channels[[i]])
		sig.channel[1,"array_index"] <- as.character(dbGetResult(res)[[1]])
		
		sig.channel[1,"sample_index"] <- as.character(sample.pks[as.numeric(sig.channel[1,"sample_index"])])
		insertIntoTable(Con,name="signal_channels",data=sig.channel,references=list(array.ref,sample.ref),no.transaction=global.transaction)
		sig.channel.pks[i] <- as.character(dbGetResult(dbSendQuery(Con,"SELECT signal_channels_pk FROM signal_channels WHERE arrays_fk=",rpgsql.format.values(sig.channel[1,"array_index"]),"AND color=",rpgsql.format.values(sig.channel[1,"color"])))[[1]])
		if(v){
			p.counter <- p.counter + 1
			progress(p.counter*100/l.c,steps=40)
		}
	}
		
	if(v){
		progress(100,steps=40)
		cat("\nfinished\n")
		cat("Inserting the expression values in the exp_values table\n")
		l.c <- length(chips)
		progress(0,steps=40)
		p.counter <- 0
	}
		#cat(paste("please wait for",length(chips),"dots:"))
	log.values <- .is.log(exprs(object))
	
	##
	# if object@weights -> ALTER TABLE exp_values ADD COLUMN weights float4
	##
	for(i in 1:length(chips)){
		exp <- exprs(object)[,i]
		data <- data.frame(id=rownames(exprs(object)),value=exp,log_value=rep(log.values,length(exp)),channel=rep(sig.channel.pks[i],length(exp)))
		insertIntoTable(Con,name="exp_values",data=data,references=new("AutoReference",source.table="exp_values",ref.table="signal_channels",source.table.column="channel",ref.table.column="signal_channels_pk"),attributes=c("id","value","log_value","channel"),no.transaction=global.transaction)
		if(v){	
			p.counter <- p.counter+1
			progress(p.counter*100/l.c,steps=40)
		}
	}
	if(v){
		progress(100,steps=40)
		cat("\nfinished\n\n")
	}
	if(global.transaction){
		r <- dbSendQuery(Con,"COMMIT TRANSACTION")
	}
}

dbGetExperimentInfo <- function(Con,exp.title=NULL){
	require(pgUtils)
	if(is.null(exp.title)){
		res <- dbSendQuery(Con,"SELECT * FROM experiments;")
		result <- dbGetResult(res)
		cat("You have not submitted an experiment title, please submit one of the following that are available in the database: \n")
	}
	else{
		exp.pk <- dbSendQuery(Con,"SELECT experiments_pk FROM experiments WHERE title=",rpgsql.format.values(exp.title))
		if(dbResultInfo(exp.pk)$error.message!=""){
			stop("No experiment with this name found! \n\n")
		}
		query1 <- paste("SELECT arrays_pk,chip FROM arrays WHERE experiments_fk=",rpgsql.format.values(dbGetResult(exp.pk)[[1]]))
		query2 <- "SELECT arrays_fk,signal_channels_pk,array_name,samples_fk,name,individual,line,tissue,treatment,in_vivo_treatments,exposure_time FROM signal_channels JOIN samples ON (samples_fk=samples_pk)"
		res <- dbSendQuery(Con,paste("SELECT * FROM (",query1,") AS first JOIN (",query2,") AS second ON (first.arrays_pk=second.arrays_fk) ORDER BY signal_channels_pk;"))
#		if(nrow(res)==0){
#			stop("Got no result!!! \n Does the experiment exist?\n\n")
#		}
		result <- dbGetResult(res)
	}
	result
}

.calculateDBRegulations.old <- function(Con,exp.title=NULL,comparisons=NULL,v=TRUE){
	#require(tkfiDB)
	if(is.null(comparisons)){
		stop("Please specify the comparisons that should be made \n\n")
	}
	if(is.null(exp.title)){
		dbGetExperimentInfo(Con,exp.title=exp.title)
	}
	else{
		experiments <- dbGetExperimentInfo(Con,exp.title=exp.title)
		dbSendQuery(Con,"BEGIN TRANSACTION;")
		table.list <- dbListTables(Con)
		
		if( !("comparisons" %in% table.list$relname)){
			if(v){
				cat("creating new comparisons table...\n")
			}
		createDBTable(Con,name="comparisons",attributes=c("r","g","description"),data.types=c("int8","int8","TEXT"),options=c("REFERENCES samples (samples_pk)","REFERENCES samples (samples_pk)",""),no.transaction=TRUE)
		}
		if( !("regulation_values" %in% table.list$relname)){
			if(v){
				cat("creating new regulation_values table...\n")
			}
			createDBTable(Con,name="regulation_values",attributes=c("id","m","a"),data.types=c("TEXT","float8","float8"),references="comparisons",no.transaction=TRUE)
		}
				
		if(class(comparisons)!="list"){
			dbSendQuery(Con,"ROLLBACK TRANSACTION;")
			stop("you should submit your comparisons as a list! also if you have only one!\n\n")
		}
		#cat(paste("Please wait for",length(comparisons),"dots (better if you go to drink some coffe, because it could take some time...):"))
		if(v){
			cat("Performing calculation\n")
			l.c <- length(comparisons)
			p.counter <- 0
			progress(0,steps=40)
		}
		for(i in 1:length(comparisons)){
			comp <- comparisons[[i]]
			if(class(comp)=="numeric" & length(comp)==2){
				write(file=log.file,paste("Creating comparison between sample",comp[1],"and",comp[2]),append=TRUE)
				##
				# now we have to check if the index does not exceed the dimensions of the experiments table...
				if(comp[1] > length(experiments[,1]) | comp[2] > length(experiments[,1])){
					dbSendQuery(con,"ROLLBACK TRANSACTION;")
					stop("Could not calculate comparisons, the desired index exceeds the number of chips available in this experiment! \n\n")
				}
				
				##
				# error checking finished (finally) now we can do the work...
				R <- dbGetResult(dbSendQuery(Con,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[comp[1],"signal_channels_pk"]),"AND log_value=FALSE;"))
								
				if(nrow(R)==0){
					R <- dbGetResult(dbSendQuery(Con,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[comp[1],"signal_channels_pk"]),"AND log_value;"))
					r <- as.matrix(R[,"value"])
					rownames(r) <- R[,"id"]
				}
				else{
					r <- as.matrix(log2(R[,"value"]))
					rownames(r) <- R[,"id"]
					rm(R)
				}
				G <- dbGetResult(dbSendQuery(Con,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[comp[2],"signal_channels_pk"]),"AND log_value=FALSE;"))
				if(nrow(G)==0){
					gd <- dbGetResult(dbSendQuery(Con,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[comp[2],"signal_channels_pk"]),"AND log_value;"))
					g <- as.matrix(gd[,"value"])
					rownames(g) <- gd[,"id"]
					ids <- gd[,"id"]
				}
				else{
					g <- as.matrix(log2(G[,"value"]))
					rownames(g) <- G[,"id"]
					ids <- G[,"id"]
					rm(G)
				}
				insertIntoTable(Con,name="comparisons",data=data.frame(r=experiments[comp[1],"samples_fk"],g=experiments[comp[2],"samples_fk"],description=paste(experiments[comp[1],"array_name"],"against",experiments[comp[2],"array_name"])),no.transaction=TRUE)
				
				##
				# ATTENTION!!!!! what happens if this three parameters are not unique? Taking the last element that was inserted...
				res <- dbGetResult(dbSendQuery(Con,"SELECT * FROM comparisons WHERE r=",rpgsql.format.values(experiments[comp[1],"samples_fk"]),"AND g=",rpgsql.format.values(experiments[comp[2],"samples_fk"]),"AND description=",rpgsql.format.values(paste(experiments[comp[1],"array_name"],"against",experiments[comp[2],"array_name"]))))
				comp.pk <- as.numeric(res[,"comparisons_pk"])
				comp.pk <- max(comp.pk)
				
				##
				# todo: calculate m and a values and insert them into the regulation_values table...
				m <- r[ids,]-g
				a <- 0.5 * (r[ids,]+g)
				Ref <- new("AutoReference",source.table="regulation_values",ref.table="comparisons",source.table.column="ref",ref.table.column="comparisons_pk")
				insertIntoTable(Con,name="regulation_values",data=data.frame(ref=rep(comp.pk,length(m)),m=m,a=a,id=ids),references=Ref,no.transaction=TRUE)
				write(file=log.file,paste("the primary key of the comparison:",comp.pk),append=TRUE)
				if(v){
					p.counter <- p.counter + 1
					progress(p.counter*100/l.c,steps=40)
				}
			}
			else{
				dbSendQuery("ROLLBACK TRANSACTION;")
				stop("don't know what to do with the comparison submitted... \n\n")
			}
			
		}
		dummy <- dbSendQuery(Con,"COMMIT TRANSACTION;")
		if(v){
			progress(100,steps=40)
			cat("\nfinished!\n")
		}
	}
}


dbCalculateRegulations <- function(Con,exp.title=NULL,comparisons=NULL,v=TRUE){
	require(pgUtils)
	if(is.null(comparisons)){
		stop("Please specify the comparisons that should be made \n\n")
	}
	if(is.null(exp.title)){
		dbGetExperimentInfo(Con,exp.title=exp.title)
	}
	else{
	
	tryCatch(.calcRegulations(Con,exp.title,comparisons,v),error=function(error){
		.rollItBack(Con)
		cat("\n",as.character(error),"\nTo get more information about the error read the log file: ",log.file,"\n")
	},interrupt=function(interrupt){
		.rollItBack(Con)
		cat("\nThe transaction was rolled back.\n")
	})
	
	#on.exit(.resetAfterError())
	#assign("ConnectionForErrors",Con,env=globalenv())
	#options(error=expression(.rollItBack(Con)))
	}
}

.calcRegulations <- function(Con,exp.title=NULL,comparisons=NULL,v=TRUE){
	experiments <- dbGetExperimentInfo(Con,exp.title=exp.title)
	dbSendQuery(Con,"BEGIN TRANSACTION;")
	table.list <- dbListTables(Con)
	
	if( !("comparisons" %in% table.list$relname)){
		if(v){
			cat("creating new comparisons table...\n")
		}
	createDBTable(Con,name="comparisons",attributes=c("r_signal_channel","g_signal_channel","description"),data.types=c("int8","int8","TEXT"),options=c("REFERENCES signal_channels (signal_channels_pk)","REFERENCES signal_channels (signal_channels_pk)",""),no.transaction=TRUE)
	}
	if( !("regulation_values" %in% table.list$relname)){
		if(v){
			cat("creating new regulation_values table...\n")
		}
		createDBTable(Con,name="regulation_values",attributes=c("id","m","a"),data.types=c("TEXT","float8","float8"),references="comparisons",no.transaction=TRUE)
	}
			
	if(class(comparisons)!="list"){
		dbSendQuery(Con,"ROLLBACK TRANSACTION;")
		stop("you should submit your comparisons as a list! also if you have only one!\n\n")
	}
	#cat(paste("Please wait for",length(comparisons),"dots (better if you go to drink some coffe, because it could take some time...):"))
	if(v){
		cat("Performing calculation\n")
		l.c <- length(comparisons)
		p.counter <- 0
		progress(0,steps=40)
	}
	for(i in 1:length(comparisons)){
		comp <- comparisons[[i]]
		if(class(comp)=="numeric" & length(comp)==2){
			write(file=log.file,paste("Creating comparison between signal channel",comp[1],"and",comp[2]),append=TRUE)
			##
			# now we have to check if the index does not exceed the dimensions of the experiments table...
			if(comp[1] > length(experiments[,1]) | comp[2] > length(experiments[,1])){
				dbSendQuery(con,"ROLLBACK TRANSACTION;")
				stop("Could not calculate comparisons, the desired index exceeds the number of chips available in this experiment! \n\n")
			}
			
			##
			# error checking finished (finally) now we can do the work...
			R <- dbGetResult(dbSendQuery(Con,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[comp[1],"signal_channels_pk"]),"AND log_value=FALSE ORDER BY exp_values_pk;"))
							
			if(nrow(R)==0){
				R <- dbGetResult(dbSendQuery(Con,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[comp[1],"signal_channels_pk"]),"AND log_value ORDER BY exp_values_pk;"))
				
				r <- as.matrix(R[,"value"])
				#if(length(R[,"id"])==length(unique(R[,"id"]))){
				rownames(r) <- R[,"id"]
				#}
			}
			else{
				r <- as.matrix(log2(R[,"value"]))
				rownames(r) <- R[,"id"]
				rm(R)
			}
			G <- dbGetResult(dbSendQuery(Con,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[comp[2],"signal_channels_pk"]),"AND log_value=FALSE ORDER BY exp_values_pk;"))
			if(nrow(G)==0){
				gd <- dbGetResult(dbSendQuery(Con,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(experiments[comp[2],"signal_channels_pk"]),"AND log_value ORDER BY exp_values_pk;"))
				g <- as.matrix(gd[,"value"])
				rownames(g) <- gd[,"id"]
				ids <- gd[,"id"]
			}
			else{
				g <- as.matrix(log2(G[,"value"]))
				rownames(g) <- G[,"id"]
				ids <- G[,"id"]
				rm(G)
			}
			insertIntoTable(Con,name="comparisons",data=data.frame(r_signal_channel=experiments[comp[1],"signal_channels_pk"],g_signal_channel=experiments[comp[2],"signal_channels_pk"],description=paste(experiments[comp[1],"array_name"],"against",experiments[comp[2],"array_name"])),no.transaction=TRUE)
			
			##
			# ATTENTION!!!!! what happens if this three parameters are not unique? Taking the last element that was inserted...
			res <- dbGetResult(dbSendQuery(Con,"SELECT * FROM comparisons WHERE r_signal_channel=",rpgsql.format.values(experiments[comp[1],"signal_channels_pk"]),"AND g_signal_channel=",rpgsql.format.values(experiments[comp[2],"signal_channels_pk"]),"AND description=",rpgsql.format.values(paste(experiments[comp[1],"array_name"],"against",experiments[comp[2],"array_name"]))))
			comp.pk <- as.numeric(res[,"comparisons_pk"])
			comp.pk <- max(comp.pk)
			
			##
			# todo: calculate m and a values and insert them into the regulation_values table...
			if(length(rownames(r))==length(unique(rownames(r)))){
				m <- r[ids,]-g
				a <- 0.5 * (r[ids,]+g)
			}
			else{
				##
				# for arrays that do not have unique identifiers like affy has
				m <- r - g
				a <- 0.5 * (r + g)
			}
			Ref <- new("AutoReference",source.table="regulation_values",ref.table="comparisons",source.table.column="ref",ref.table.column="comparisons_pk")
			insertIntoTable(Con,name="regulation_values",data=data.frame(ref=rep(comp.pk,length(m)),m=m,a=a,id=ids),references=Ref,no.transaction=TRUE)
			write(file=log.file,paste("the primary key of the comparison:",comp.pk),append=TRUE)
			if(v){
				p.counter <- p.counter + 1
				progress(p.counter*100/l.c,steps=40)
			}
		}
		else{
			dbSendQuery(Con,"ROLLBACK TRANSACTION;")
			stop("don't know what to do with the comparison submitted... \n\n")
		}
		
	}
	dummy <- dbSendQuery(Con,"COMMIT TRANSACTION;")
	if(v){
		progress(100,steps=40)
		cat("\nfinished!\n")
	}
}


getSamplesFromDB <- function(Con,table.name="samples",pattern=NULL){
	require(pgUtils)
	if(is.null(pattern)){
		res <- dbSendQuery(Con,"SELECT * FROM ",table.name,";")
		if(dbResultInfo(res)$error.message!=""){
			stop(paste("An error occurred: ",dbResultInfo(res)$error.message))
		}
		result <- dbGetResult(res)
	}
	else{
		result <- searchIn(Con,name=table.name,pattern=pattern)
	}
	result
}



## no help yet
getMDB <- function(con,ids=NULL,comparisons.pk=NULL,v=TRUE,column.names="description"){
	require(pgUtils)
	if(is.null(ids)){
		stop("Please submit the IDs for which you would get the m values (ids=)\n\n")
	}
	if(is.null(comparisons.pk)){
		#if(v){
		#	cat("")
		#}
		#info <- dbGetResult(dbSendQuery(con,"SELECT * FROM comparisons JOIN samples ON (r=samples_pk)"))
		#info.sub <- .reduceInfo(info,include.by.name=include.by.name,include.by.pk=include.by.pk,exclude.by.name=exclude.by.name,exclude.by.pk=exclude.by.pk)
		#comparisons.pk <- info.sub[,"comparisons_pk"]

		stop("Please submit the comparisons \n\n")
	}
	m.table <- matrix(ncol=(length(comparisons.pk)+1),nrow=length(ids))
	m.table[,1] <- ids
	c.names <- "UniqueID"
	if(v){
		counter <- 0
		l.c <- length(comparisons.pk)
		cat("Retrieving data from the database\n")
		progress(0,steps=40)
	}
	#cat(paste("Please wait for",length(comparisons.pk),"dots:"))
	for(i in 1:length(comparisons.pk)){
		if(v){
			counter <- counter+1
			progress(counter*100/l.c,steps=40)
		}
		comp <- dbGetResult(dbSendQuery(con,"SELECT * FROM comparisons JOIN signal_channels ON (r_signal_channel=signal_channels_pk) JOIN samples ON (samples_fk=samples_pk) WHERE comparisons_pk=",rpgsql.format.values(comparisons.pk[i])))
		if(nrow(comp)>0){
			if(length(ids)>200){
				res <- dbGetResult(dbSendQuery(con,"SELECT m,id FROM regulation_values WHERE comparisons_fk=",rpgsql.format.values(comparisons.pk[i])," ORDER BY regulation_values_pk"))
			}
			else{
				res <- dbGetResult(dbSendQuery(con,"SELECT m,id FROM regulation_values WHERE comparisons_fk=",rpgsql.format.values(comparisons.pk[i]),"AND id IN (",list.to.csv(rpgsql.format.values(ids)),") ORDER BY regulation_values_pk"))
			}
			c.names <- c(c.names,list.to.csv(comp[1,column.names]))
			dummy <- matrix(ncol=1,nrow=nrow(res))
			dummy[,1] <- res[,"m"]
			if(length(res[,"id"])==length(unique(res[,"id"]))){
				rownames(dummy) <- res[,"id"]
				m.table[,(i+1)] <- dummy[ids,]
				dummy.ids <- ids
			}
			else{
				idx <- multiGrep(res[,"id"],pattern=ids,exact.match=TRUE)
				if(i==1){
					m.table <- matrix(ncol=(length(comparisons.pk)+1),nrow=length(idx))
					m.table[,1] <- res[idx,"id"]
					dummy.ids <- res[idx,"id"]
				}
				
				m.table[,(i+1)] <- res[idx,"m"]
			}
		}
		else{
			c.names <- c(c.names,"no comparison")
		}
		#cat(".")
	}
	if(v){
		progress(100,steps=40)
		cat("\nfinished\n")
	}
	colnames(m.table) <- c.names
	rownames(m.table) <- dummy.ids
	m.table
}




##
# reads out the expression values from selected signal channels...
# new: if number of signal channels < 200 -> do 1 query for all!
getEDB <-  function(con,ids=NULL,signal.channels.pk=NULL,column.names=c("array_name","color"),v=TRUE){
	require(pgUtils)
	if(is.null(ids)){
		stop("Please submit the ids of the genes for which you want to get the expression values (ids=) \n\n")
	}
	if(is.null(signal.channels.pk)){
		#if(v){
		#	cat("")
		#}
		#info <- dbGetResult(dbSendQuery(con,"SELECT * FROM signal_channels JOIN samples ON (samples_fk=samples_pk)"))
		#info.sub <- .reduceInfo(info,include.by.name=include.by.name,include.by.pk=include.by.pk,exclude.by.name=exclude.by.name,exclude.by.pk=exclude.by.pk)
		#signal.channels.pk <- info.sub[,"signal_channels_pk"]
		
		stop("Please submit the primary keys for the signal_channels. \n\n")
	}

	## check if signal channels are in the database!
	old.sch <- signal.channels.pk
	signal.channels.pk <- as.character(dbGetResult(dbSendQuery(con,"SELECT signal_channels_pk FROM signal_channels WHERE signal_channels_pk IN (" ,paste(rpgsql.format.values(signal.channels.pk) ,collapse=","), ")"))[, "signal_channels_pk"] )
	if(length(old.sch) != length(signal.channels.pk)){
		if(v){
			cat(paste((length(old.sch)-length(signal.channels.pk)),"of the",length(old.sch),"signal channels can not be found in the database!\n"))
		}
		warning(paste("Not all submitted signal channels could be found in the database!",(length(old.sch)-length(signal.channels.pk)),"of the",length(old.sch)," submitted signal channels could not be found\n"))
	}

	if(length(signal.channels.pk)<250){
		## doing all in one call...
		if(length(ids)>50){
			## retrieving all, subsetting in R...
			Result <- dbGetResult(dbSendQuery(con,"SELECT id, value, signal_channels_pk, ",paste(column.names,collapse=",")," FROM (SELECT id,value,signal_channels_fk FROM exp_values WHERE signal_channels_fk IN (",paste(rpgsql.format.values(signal.channels.pk),collapse=","),")) AS dummy JOIN signal_channels ON (signal_channels_fk=signal_channels_pk) JOIN samples ON (samples_fk=samples_pk) ORDER BY signal_channels_pk,id"))
			## now get only the interesting ones...
			Result <- Result[ Result[,"id"] %in% ids, ]	## should now have the same as below (preserves also the ordeing i got from the query...)...
		}
		else{
			Result <- dbGetResult(dbSendQuery(con,"SELECT id, value, signal_channels_pk, ",paste(column.names,collapse=",")," FROM (SELECT id,value,signal_channels_fk FROM exp_values WHERE signal_channels_fk IN (",paste(rpgsql.format.values(signal.channels.pk),collapse=","),") AND id IN (", paste(rpgsql.format.values(ids) ,collapse=","), ")) AS dummy JOIN signal_channels ON (signal_channels_fk=signal_channels_pk) JOIN samples ON (samples_fk=samples_pk) ORDER BY signal_channels_pk,id"))
		}
		## check if the ordering of the ids is the same for all signal channels (should always be the case since we order it in the SQL query!)
		ids_ordered = as.character(Result[ Result[ ,"signal_channels_pk"] == signal.channels.pk[1] , "id"])
#		return (Result)
		if( sum(Result[,"id"] == rep(ids_ordered, length(signal.channels.pk) ) ) == nrow(Result)){
			## everything ok! can do a RES <- matrix(Result, ncol=...)
			ToReturn <- matrix(Result[,"value"],ncol=length(signal.channels.pk))
			rownames(ToReturn) <- ids_ordered
			CN <- unique(Result[,column.names])
			if(length(column.names) > 1){
				CN <- apply(CN,MARGIN=1,paste,collapse=" ")
			}
			colnames(ToReturn) <- CN
			return(ToReturn)
		}
		else{
			##damned, have to loop through!
			#cat("damned... loop \n")
			ToReturn <- matrix(nrow=length(ids_ordered),ncol=length(signal.channels.pk))
			rownames(ToReturn) <- ids_ordered
			for(sigch in 1:length(signal.channels.pk)){
				cat("doing",sigch)
				dummy <- Result[Result[,"signal_channels_pk"]==signal.channels.pk[sigch], ]
				ToReturn[ as.character(dummy[,"id"]) ,sigch] <- dummy[,"value"]
			}
			CN <- unique(Result[,column.names])
			if(length(column.names) > 1){
				CN <- apply(CN,MARGIN=1,paste,collapse=" ")
			}
			colnames(ToReturn) <- CN
			return(ToReturn)
		}
	}
	else{
	## "old" version...
		exp.table <- matrix(ncol=length(signal.channels.pk),nrow=length(ids))
		if(v){
			l.s <- length(signal.channels.pk)
			counter <- 0
			cat("Retrieving data from the database\n")
			progress(0,steps=40)
		}
		#cat(paste("please wait for",length(signal.channels.pks),"dots:"))
		col.names <- NULL
		for(i in 1: length(signal.channels.pk)){
			if(v){
				counter <- counter+1
				progress(counter*100/l.s,steps=40)
			}
			name <- "unknown"
			## because i am afraid of for loops in R (they are definiteley not fast) we are getting all the ids from 
			## one signal channel and subset this using R
			sig.channels <- dbGetResult(dbSendQuery(con,"SELECT * FROM signal_channels JOIN samples ON (samples_fk=samples_pk) WHERE signal_channels_pk=",rpgsql.format.values(signal.channels.pk[i])))
			if(nrow(sig.channels)>0){
				## got a signal channel...
				#name <- paste(sig.channels[1,"array_name"],sig.channels[1,"color"])
				name <- list.to.csv(sig.channels[1,column.names])
				if(length(ids)>100){
					exps <- dbGetResult(dbSendQuery(con,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(signal.channels.pk[i])," ORDER BY exp_values_pk"))
				}
				else{
					
					exps <- dbGetResult(dbSendQuery(con,"SELECT id,value FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(signal.channels.pk[i]),"AND id IN (",list.to.csv(rpgsql.format.values(ids)),") ORDER BY exp_values_pk"))
				
				}
				if(nrow(exps)>0){
					if(length(exps[,"id"])==length(unique(exps[,"id"]))){
						dummy <- matrix(ncol=1,nrow=nrow(exps))
						rownames(dummy) <- exps[,"id"]
						dummy[,1] <- exps[,"value"]
						exp.table[,i] <- dummy[ids,1]
						if(i==1){
							dummy.ids <- ids
						}
					}
					else{
						##
						# the ids are not unique, this makes it a little more difficult...
						idx <- multiGrep(exps[,"id"],pattern=ids,exact.match=TRUE)
						if(i==1){
							exp.table <- matrix(ncol=length(signal.channels.pk),nrow=length(idx))
							dummy.ids <- exps[idx,"id"]
					}
						exp.table[,i] <- exps[idx,"value"]
					}
				}
			}
			col.names <- c(col.names,name)
			#cat(".")
		}
		colnames(exp.table) <- col.names
		rownames(exp.table) <- dummy.ids
		if(v){
			progress(100,steps=40)
			cat("\nfinished\n")
		}
		return(exp.table)
	}
}


##################################################################################################
## THIS FUNCTION HAS TO BE TESTED!!!!!!
#####
deleteExperimentFromDB <- function(con=NULL,exp.title=NULL,v=TRUE){
	require(pgUtils)
	if(is.null(con)){
		stop("Please submit the connection to the database with the con attribute!\n\n")
	}
	if(is.null(exp.title)){
		stop("You forgot to specify what experiment should be deleted! (i guess i should not delete all, or should i? ;) )\n\n")
	}
	dbSendQuery(con,"BEGIN TRANSACTION")
	
	tryCatch(.deleteTheExperiment(con,exp.title,v),error=function(error){
		.rollItBack(con)
		cat("\n",as.character(error),"\nTo get more information about the error read the log file: ",log.file,"\n")
	},interrupt=function(interrupt){
		.rollItBack(con)
		cat("\nThe transaction was rolled back.\n")
	})
	#assign("ConnectionForErrors",Con,env=globalenv())
	#options(error=expression(.rollItBack(con)))
	#on.exit(dbSendQuery(con, "ROLLBACK TRANSACTION"))
	#on.exit(.resetAfterError())
	

}


.deleteTheExperiment <- function(con,exp.title,v){
	exp.table <- dbGetExperimentInfo(con,exp.title)
	table.list <- dbListTables(con)
	#cat(paste("Deleting experiment",exp.title,"from the database!\n please wait for",nrow(exp.table),"points:"))
	if(v){
		cat(paste("Deleting experiment",exp.title,"from the database!\n"))
		p.counter <- 0
		l.e <- nrow(exp.table)
		#progress(0,steps=40)
	}
	##deleting regulation values...
	if( ("comparisons" %in% table.list$relname)){
		reg.info <- dbGetResult(dbSendQuery(con,"SELECT * FROM comparisons"))
		reduced.reg.info <- reg.info[c(multiGrep(reg.info[,"r_signal_channel"],pattern=exp.table[,"signal_channels_pk"],exact.match=TRUE),multiGrep(reg.info[,"g_signal_channel"],pattern=exp.table[,"signal_channels_pk"],exact.match=TRUE)),]
		
		if(length(reduced.reg.info[,1])>0){
			for(i in 1:length(reduced.reg.info[,1])){
				res <- dbSendQuery(con,"DELETE FROM regulation_values WHERE comparisons_fk=",rpgsql.format.values(reduced.reg.info[i,"comparisons_pk"]))
				resInfo <- dbResultInfo(res)
				if(resInfo$error.message!=""){
					if(do.log){
						loggit(res,call=paste("ERROR: ","DELETE FROM regulation_values WHERE comparisons_fk=",rpgsql.format.values(reduced.reg.info[i,"comparisons_pk"])))
					}
					dbSendQuery(con,"ROLLBACK TRANSACTION")
					stop("An error occurred! See the log file (by default tkfiDB.log in the working directory)!")
				}
				cat("regulation_values table:",resInfo$command.response,"\n")

				res <- dbSendQuery(con,"DELETE FROM comparisons WHERE comparisons_pk=",rpgsql.format.values(reduced.reg.info[i,"comparisons_pk"]))
				resInfo <- dbResultInfo(res)
				if(resInfo$error.message!=""){
					if(do.log){
						loggit(res,call=paste("ERROR: ","DELETE FROM comparisons WHERE comparisons_pk=",rpgsql.format.values(reduced.reg.info[i,"comparisons_pk"])))
					}
					dbSendQuery(con,"ROLLBACK TRANSACTION")
					stop("An error occurred! See the log file (by default tkfiDB.log in the working directory)!")
				}
				cat("comparisons table:",resInfo$command.response,"\n")
			}
		}
	}
	
	
	for(i in 1:nrow(exp.table)){
		res <- dbSendQuery(con,"DELETE FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(exp.table[i,"signal_channels_pk"]))
		resInfo <- dbResultInfo(res)
		if(resInfo$error.message!=""){
			if(do.log){
				loggit(res,call=paste("ERROR: ","DELETE FROM exp_values WHERE signal_channels_fk=",rpgsql.format.values(exp.table[i,"signal_channels_pk"])))
			}
			dbSendQuery(con,"ROLLBACK TRANSACTION")
			stop("An error occurred! See the log file (by default tkfiDB.log in the working directory)!")
		}
		cat("exp_values table:",resInfo$command.response,"\n")
		
		res <- dbSendQuery(con,"DELETE FROM signal_channels WHERE signal_channels_pk=",rpgsql.format.values(exp.table[i,"signal_channels_pk"]))
		resInfo <- dbResultInfo(res)
		if(resInfo$error.message!=""){
			if(do.log){
				loggit(res,call=paste("ERROR: ","DELETE FROM signal_channels WHERE signal_channels_pk=",rpgsql.format.values(exp.table[i,"signal_channels_pk"])))
			}
			dbSendQuery(con,"ROLLBACK TRANSACTION")
			stop("An error occurred! See the log file (by default tkfiDB.log in the working directory)!")
		}
	}
	
	arrays.pks <- unique(exp.table[,"arrays_pk"])
	for(i in 1:length(arrays.pks)){
		res <- dbSendQuery(con,"DELETE FROM arrays WHERE arrays_pk=",rpgsql.format.values(arrays.pks[i]))
		resInfo <- dbResultInfo(res)
		if(resInfo$error.message!=""){
			if(do.log){
				loggit(res,call=paste("ERROR: ","DELETE FROM arrays WHERE arrays_pk=",rpgsql.format.values(arrays.pks[i])))
			}
			dbSendQuery(con,"ROLLBACK TRANSACTION")
			stop("An error occurred! See the log file (by default tkfiDB.log in the working directory)!")
		}
		cat("arrays table:",resInfo$command.response,"\n")
		#if(v){
		#	p.counter <- p.counter + 1
		#	progress(p.counter*100/l.e,steps=40)
		#}
	}
	
	result <- dbGetResult(dbSendQuery(con,"SELECT * FROM experiments WHERE title=",rpgsql.format.values(exp.title)))
	res <- dbSendQuery(con,"DELETE FROM experiments WHERE experiments_pk=",rpgsql.format.values(result[1,"experiments_pk"]))
	resInfo <- dbResultInfo(res)
	if(resInfo$error.message!=""){
		if(do.log){
			loggit(res,call=paste("ERROR: ","DELETE FROM experiments WHERE experiments_pk=",rpgsql.format.values(result[1,"experiments_pk"])))
		}
		dbSendQuery(con,"ROLLBACK TRANSACTION")
		stop("An error occurred! See the log file (by default tkfiDB.log in the working directory)!")
	}

	#if(v){
	#	progress(100,steps=40)
	#	cat("\nfinished!\n\n")
	#}
	dbSendQuery(con,"COMMIT TRANSACTION")

}


.reduceInfo <- function(info,include.by.name=NULL,include.by.pk=NULL,exclude.by.name=NULL,exclude.by.pk=NULL,pk.col="comparisons_pk",order=FALSE){
	if(!is.null(exclude.by.name)){
		bm <- rep(TRUE,length(info[,1]))
		idx <- multiGrep(info[,"name"],pattern=exclude.by.name,exact.match=TRUE)
		if(length(idx)>0){
			bm[idx] <- FALSE
		}
		info <- info[bm,]
		if(min(dim(info))==0){
			stop("Sorry, nothing left to do. All available chips where excluded due your restrictions!\n\n")
		}
	}

	#remove replicates
	if(!is.null(exclude.by.pk)){
		bm <- rep(TRUE,length(info[,1]))
		idx <- multiGrep(info[,pk.col],pattern=exclude.by.pk,exact.match=TRUE)
		if(length(idx)>0){
			bm[idx] <- FALSE
		}
		info <- info[bm,]
	}
	if(!is.null(include.by.name)){
		bm <- rep(FALSE,length(info[,1]))
		idx <- multiGrep(info[,"name"],pattern=include.by.name,exact.match=TRUE)
		if(length(idx)==0){
			stop("Sorry, nothing left to do. All available chips where excluded due your restrictions!\n\n")
		}
		if(order){
			info <- info[idx,]
		}
		else{
			if(length(idx)>0){
				bm[idx] <- TRUE
			}
			info <- info[bm,]
		}
	}
	if(!is.null(include.by.pk)){
		bm <- rep(FALSE,length(info[,1]))
		idx <- multiGrep(info[,pk.col],pattern=include.by.pk,exact.match=TRUE)
		if(length(idx)==0){
			stop("Sorry, nothing left to do. All available chips where excluded due your restrictions!\n\n")
		}
		if(order){
			info <- info[idx,]
		}
		else{
			if(length(idx)>0){
				bm[idx] <- TRUE
			}
			info <- info[bm,]
		}
	}
	info
}

##
## gets the comparisons from the database.
dbGetComparisons <- function(con,exp.title=NULL,join.tables=c("signal_channels","samples","arrays","experiments")){
	require(pgUtils)
	red <- dbGetResult(dbSendQuery(con,createJOINQuery(join="comparisons",with="experiments",r=TRUE),"JOIN samples ON (samples_fk=samples_pk) ORDER BY comparisons_pk"))
	green <- dbGetResult(dbSendQuery(con,createJOINQuery(join="comparisons",with="experiments",r=FALSE),"JOIN samples ON (samples_fk=samples_pk) ORDER BY comparisons_pk"))
	if(!is.null(exp.title)){
		red <- red[red[,"title"]==exp.title,]
		green <- green[green[,"title"]==exp.title,]
		if(nrow(red)==0 | nrow(green)==0){
			stop("For this experiment there are no comparisons in the database!\n\n")
		} 
	}
	
	## ... remove all "description" columns
	red <- red[,!colnames(red)=="description"]
	green <- green[,!colnames(green)=="description"]
	
	Comp.ColNames <- dbColnames(con,"comparisons")
	
	## remove description...
	Comp.ColNames <- Comp.ColNames[Comp.ColNames!="description"]
	
	ColumnNames <- Comp.ColNames
	GetCN <- NULL
	if(length(join.tables)>0){
		for(i in 1:length(join.tables)){
			CN <- dbColnames(con,join.tables[i])
			# remove "description columns"
			CN <- CN[CN!="description"]
			GetCN <- c(GetCN,CN)
			ColumnNames <- c(ColumnNames,paste(CN,"red",sep="."))
			ColumnNames <- c(ColumnNames,paste(CN,"green",sep="."))
		}
	}
	Result <- matrix(ncol=length(ColumnNames),nrow=nrow(red))
	colnames(Result) <- ColumnNames
	Result[,Comp.ColNames] <- as.matrix(red[,Comp.ColNames])
	if(length(GetCN)>0){
		GetCN <- unique(GetCN)
		for(i in 1:length(GetCN)){
			Result[,colnames(Result)==paste(GetCN[i],"red",sep=".")] <- red[,colnames(red)==GetCN[i]]
			Result[,colnames(Result)==paste(GetCN[i],"green",sep=".")] <- green[,colnames(green)==GetCN[i]]
		}
	}
	Result
}

createJOINQuery <- function(join,with,select="*",r=TRUE){
	query <- ""
	##
	#  comparisons table...
	if(join=="comparisons" & with=="signal_channels"){
		if(r){
			query <- paste("SELECT",select,"FROM comparisons JOIN signal_channels ON (r_signal_channel=signal_channels_pk)")
		}
		else{
			query <- paste("SELECT",select,"FROM comparisons JOIN signal_channels ON (g_signal_channel=signal_channels_pk)")
		}
	}
	if(join=="comparisons" & with=="samples"){

		query <- paste(createJOINQuery(join,with="signal_channels",select=select,r=r),"JOIN samples ON (samples_fk=samples_pk)")
	}
	if(join=="comparisons" & with=="arrays"){
		query <- paste(createJOINQuery(join,with="signal_channels",select=select,r=r),"JOIN arrays ON (arrays_fk=arrays_pk)")
	}
	if(join=="comparisons" & with=="experiments"){
		query <- paste(createJOINQuery(join,with="arrays",select=select,r=r),"JOIN experiments ON (experiments_fk=experiments_pk)")
	}
	
	##
	#  signal_channels_table
	if(join=="signal_channels" & with=="samples"){
		query <- paste("SELECT",select,"FROM signal_channels JOIN samples ON (samples_fk=samples_pk)")
	}
	if(join=="signal_channels" & with=="arrays"){
		query <- paste("SELECT",select,"FROM signal_channels JOIN arrays ON (arrays_fk=arrays_pk)")
	}
	if(join=="signal_channels" & with=="experiments"){
		query <- paste(createJOINQuery(join,with="arrays",select=select,r=r),"JOIN experiments ON (experiments_fk=experiments_pk)")
	}
	query
}

.rollItBack <- function(Con){
	#if(exists("ConnectionForErrors",envir=globalenv())){
	if(dbConnectionInfo(Con)$status==0){
		dbSendQuery(Con,"ROLLBACK TRANSACTION")
	}
	#}
}

.resetAfterError <- function(){
	options(error=NULL)
	rm(ConnectionForErrors,inherits=TRUE)
}

.processColumnNames <- function(column.names=NULL){
	splitted <- strsplit(column.names,split=".",extended=FALSE,fixed=TRUE)
	col.names <- unique(unlist(lapply(splitted,function(x){ret <- x[length(x)]})))
	table.names <- unique(unlist(lapply(splitted,function(x){if(length(x)>1){ret <- x[1]}})))
	result <- data.frame(sql.colnames=column.names,colnames=col.names,tablenames=table.names)
	result
}

