#######################################################################################################
#######################################################################################################
##
##  CORE FUNCTIONS
##
#######################################################################################################

########################################################################
## calculates the mean of all submitted values except the NAs
########################################################################
meanWithoutNA <- function(Data){

mean <- (sum(Data[!is.na(Data)]) / length(Data[!is.na(Data)]))
mean
}


########################################################################
## calculates the median of all submitted values except the NAs
########################################################################
medianWithoutNA <- function(Data){

median <- median(Data[!is.na(Data)])

}

########################################################################
######## My own grep function				################
########################################################################
myGrep <- function(pattern, x,exact.match=FALSE,...){
	if(is.character(x)){
		if(exact.match){
			result <- grep("TRUE",as.character(x==pattern))
		}
		else{
			result <- grep(pattern,x,...)
		}
	}
	else{
		if(is.null(dim(x))){
			if(exact.match){
				result <- grep("TRUE",as.character(x==pattern))
			}
			else{
				result <- grep(pattern,as.character(x),...)
			}
		}
		else{
			if(exact.match){
				result <- grep("TRUE",as.character(x==pattern))
			}
			else{
				result <- grep(pattern,as.character(x[,]),...)
			}
		}
	}
	result
}

########################################################################
######## make a grep from a list of input values...
########################################################################
multiGrep <- function(pattern,x,...){
	result <- NULL
	pattern <- unique(pattern)
	if(length(pattern)>0){
		for(i in 1:length(pattern)){
			result <- c(result,myGrep(pattern[i],x,...))
		}
	}
	else{
		result <- myGrep(pattern,x,...)
	}
	result
}

#grepp <- function()

########################################################################
## calcualtes the G values from the M and A
########################################################################
MAtoG <- function(M,A){
	sqrt((2^(2*A))/2^M)
}

########################################################################
## calcualtes the G values from the M and A
########################################################################
MAtoR <- function(M,A){
	MAtoG(M,A) * 2^M
}


###########################################################################
##
###########################################################################
.similar <- function(a,b){
	counter <- 0
	for(i in 1:length(a)){
		counter <- counter + sum((a[i]==b)==TRUE)
	}
	counter
}


###########################################################################
##
###########################################################################
#getFromMatrix <- function(bm, matrix){
#	if(length(bm)!=length(matrix[,1])){
#		stop("Boolean mask must have the same length as matrix column length! \n")
#	}
#	result <- data.frame(matrix(ncol=length(matrix[1,]),nrow=sum(bm==TRUE)))
#	colnames(result) <- colnames(matrix)
#	for(i in 1:length(matrix[1,])){
#		result[,i] <- matrix[,i][bm]
#	}
#	result
#}


###########################################################################
## checks if the value is a decimal number (e.g. 0.2) or a integer (e.g. 2).
###########################################################################
isDecimal <- function(x){
	bool <- ceiling(x) != floor(x)
	bool
}


replaceAll <- function(x,old,new){
	result <- x
	for(i in 1:length(x)){
		result[i] <- replaces(x=x[i],old,new)
	}
	result
}

###########################################################################
## replaces 
###########################################################################
.replaces <- function(x,old,new){
	# split x into single characters
	if(length(old)>length(new)){
		cat(paste("Lenght of vector old differs from vector new. Replacing all old characters with ",new[1],"\n"))
		newnew <- matrix(ncol=1,nrow=length(old))
		newnew[,] <- new[1]
		new <- newnew
	}
	#if(length(old)>1){
	#	cat(paste("taking only the first element and replacing all",old[1],"with",new,"\n"))
	#	old <- old[1]
	#}
	result <- x
	for(j in 1:length(old)){
		dummy <- strsplit(result,split="")
		dummy[[1]][dummy[[1]]==old[j]] <- new[j]
		dummy2 <- NULL
		for(i in 1:length(dummy[[1]])){
			dummy2 <- paste(dummy2,dummy[[1]][i],sep="")
		}
		result <- dummy2
	}
	#dummy2
	result
}

replaces <- function(x,old,new,...){
	if(length(old)>length(new)){
		cat(paste("Lenght of vector old differs from vector new. Replacing all old characters with ",new[1],"\n"))
		newnew <- matrix(ncol=1,nrow=length(old))
		newnew[,] <- new[1]
		new <- newnew
	}
	for(i in 1:length(old)){
		if(old[i]=="\\" | length(unlist(strsplit(old[i],split="")))==1){
			dummy <- .replaces(x=x,old=old[i],new=new[i])
		}
		else{
			dummy <- strsplit(x,old[i],...)[[1]]
			#cat(dummy,"\n")
			if(length(dummy)>1){
				nr.splits <- length(dummy)
				dummy <- paste(dummy,collapse=new[i])
				if(nchar(x)!=(nchar(dummy)-((nr.splits-1)*nchar(new[i]))+((nr.splits-1)*nchar(old[i])))){
					dummy <- paste(dummy,new[i],sep="")
				}
				#dummy <- paste(c(paste(dummy[1:(length(dummy)-1)],new[i],sep=""),dummy[length(dummy)]),collapse="")
			}
			else{
				if(nchar(dummy)!=nchar(x)){
					dummy <- paste(dummy,new[i],sep="")
				}
			}
		}
		x <- dummy
	}
	#dummy
	x
}

########################################################################################################################
########################################################################################################################
##		LATEX stuff
##
########################################################################################################################

########################################################################################################################
## writing data as table into a tex document. other (better) possibility: use the xtable library!
########################################################################################################################
writeLatexTable <- function(data=NULL,file=NULL,append=TRUE,textsize="normalsize",caption=NULL,label=NULL,digits=NULL){
	if(is.null(file)){
		stop("no file submitted! submit or a file handle or a filename (with relative or absolute path. Don't panic, the new information will be appended to the existing file if the file does already exist :) )\n")
	}
	if(is.null(data)){
		stop("That's fine, there is nothing to write... can have a coffee break ;) \n please submit the table (matrix or whatever).\n")
	}
	have.to.close <- FALSE
	if(is.character(file)){
		path <- getPathFromFile(file)
		createDirs(path)
		if(append){
			file <- file(file,"a")
		}
		else{
			file <- file(file,"w")
		}
		have.to.close <- TRUE
	}
	if(is.null(dim(data))){
		stop("You have not submitted a two dimensional data table!\n")
	}
	
	cat("\n% table was created automatically by the R script writeLatexTable \n\n",file=file)
	cat("\\begin{table}[ht]\n",file=file)
	cat("\\begin{center}\n",file=file)
	cat(paste("\\begin{",textsize,"}\n",sep=""),file=file)
	names <- colnames(data)
	r.names <- rownames(data)
	
	if(is.null(names)){
		names <- as.character(seq(1,dim(data)[2]))
	}
	if(is.null(r.names)){
		r.names <- as.character(seq(1,dim(data)[1]))
	}
	
	names <- replaceAll(names,old="_",new=" ")
	#r.names <- replaceAll(as.character(r.names),old=c("_")," ")
	names <- format.latex(names)
	r.names <- format.latex(r.names)
	
	line <- NULL
	for(i in 1:length(data[1,])){
		line <- paste(line,"r",sep="")
	}
	line <- paste("|l|",line,"|",sep="")
	
	cat(paste("\\begin{tabular}{",line,"}\n",sep=""),file=file)
	cat("\\hline\n",file=file)
	
	#the header
	line <- " "
	for(i in 1:length(names)){
		line <- paste(line,names[i],sep=" & ")
	}
	line <- paste(line,"\\\\\n")
	cat(line,file=file)
	cat("\\hline\n",file=file)
	
	#writing the content...
	for(i in 1:length(data[,1])){
		line <- r.names[i]
		for(j in 1:length(names)){
			if(is.na(data[i,j])){
				line <- paste(line," &  ",sep="")
			}
			else{
				value <- as.character(data[i,j])
				if(!is.na(as.numeric(value))){
					value <- format(as.numeric(value),digits=digits)
				}
				else{
					value <- format.latex(value)
				}
				line <- paste(line," & ",value,sep="")
			}
		}
		cat(replaceAll(paste(line,"\\\\\n"),old="_",new=" "),file=file)
	}
	
	cat("\\hline\n",file=file)
	cat("\\end{tabular}\n",file=file)
	if(!is.null(caption)){
		cat(paste("\\caption{",caption,"}\n",sep=""),file=file)
	}
	if(!is.null(label)){
		cat(paste("\\label{",label,"}\n",sep=""),file=file)
	}
	cat(paste("\\end{",textsize,"}\n",sep=""),file=file)
	cat("\\end{center}\n",file=file)
	cat("\\end{table}\n",file=file)
	if(have.to.close){
		close(file)
	}
}


####################################################################################################
##
####################################################################################################
format.latex <- function(x,...){
	#cat("am here \n")
	x <- as.character(x)
	x <- replaceAll(x,old="%",new="\\%")
	x <- replaceAll(x,old="$",new="\\$")
	x <- replaceAll(x,old="_",new="\\_")
	x <- replaceAll(x,old="&",new="\\&")
	x <- replaceAll(x,old="{",new="\\{")
	x <- replaceAll(x,old="}",new="\\}")

	x
}



####################################################################################################
## write a figure in a tex document
####################################################################################################
writeFigure <- function(file=NULL,image=NULL,label=NULL,caption=NULL,option="[!ht]",image.width=1){
        #name <- replaceAll(name,old=c("#"," ","%","_"),"")
	if(is.null(file)){
		stop("no file submitted! submit or a file handle or a filename (with relative or absolute path. Don't panic, the new information will be appended to the existing file if the file does already exist :) )\n")
	}
	if(is.null(image)){
		stop("no image file submitted! \n")
	}
	have.to.close <- FALSE
	if(is.character(file)){
		file <- file(file,"a")
		have.to.close <- TRUE
	}
	cat("% image was drawn and included automatically by the R script writeFigure.\n",file=file)
	cat("\n",file=file)
	cat(paste("\\begin{figure}",option," \n",sep=""),file=file)
	cat("\t \\begin{center} \n",file=file)
	
	cat(paste("\t \\includegraphics[width=",image.width,"\\textwidth]","{",image,"} \n",sep=""),file=file)
		
	if(!is.null(caption)){
		cat(paste("\t \\caption{",caption,"} \n"),file=file)
	}
	if(!is.null(label)){
		cat(paste("\t \\label{",label,"} \n"),file=file)
	}

	
	cat("\t \\end{center} \n",file=file)
	cat("\\end{figure} \n",file=file)
	cat("\n",file=file)
	if(have.to.close){
		close(file)
	}
}


######################################################################################################
##
######################################################################################################
createFile <- function(file){
#	cat(paste("createFile: ",file))
	file <- replaceAll(file,old="\\",new="/")
	splitted <- strsplit(file,"/")
	splitted <- splitted[[1]]
	dummy <- ""
	if(length(splitted)>1){
		for(i in 1:(length(splitted)-1)){
			
			dummy <- paste(dummy,splitted[i],"/",sep="")
		}
		createDirs(dummy)
		if(!file.exists(file)){
			file.create(file)
		}
	}
	else{
		if(!file.exists(dummy)){
			file.create(dummy)
		}
	}
}



######################################################################################################
##
######################################################################################################
createDirs <- function(dir){
#	cat(paste("createDirs: ",file))
	file <- dir
	file <- replaceAll(file,old="\\",new="/")
	splitted <- strsplit(file,"/")
	splitted <- splitted[[1]]
	dummy <- ""
	if(length(splitted)>1){
		for(i in 1:(length(splitted))){
			dummy <- paste(dummy,splitted[i],"/",sep="")
			if(!file.exists(dummy)){
				dir.create(dummy)
			}
		}
	}
	else{
		if(!file.exists(dummy)){
			dir.create(file)
		}
	}
}





######################################################################################################
##
######################################################################################################
getPathFromFile <- function(file){
#	cat(paste("getPathFromFile: ",file))
	file <- replaceAll(file,old="\\",new="/")
	splitted <- strsplit(file,"/")
	splitted <- splitted[[1]]
	dummy <- ""
	if(length(splitted)>1){
		for(i in 1:(length(splitted)-1)){
			
			dummy <- paste(dummy,splitted[i],"/",sep="")
		}
		path <- dummy
	}
	else{
		path <- ""
	}
	path
}


######################################################################################################
## creates a LaTeX document from a template given. 
######################################################################################################
.createLatexFromTemplate <- function(file=NULL,template="latextemplate.tgz",title=NULL,author=NULL,includes=NULL){
	if(is.null(file)){
		stop("You must submit the file name for the LaTeX document! \n")
	}
	cat(paste("Creating the LaTex document from the template file: ",template,"\n If the file does not exist, or the path is incorrect you will get an error ;) \n"))
	input <- gzfile(template)
	content <- readLines(input)
	close(input)
	
	content[1] <- paste("%",content[1])
	createDirs(getPathFromFile(file))
	if(!is.null(title)){
		content[content=="@title"] <- paste("\\title{",title,"}",sep="")
	}
	else{
		content[content=="@title"] <- "\\title{unknown document}"
	}
	if(!is.null(author)){
		content[content=="@author"] <- paste("\\author{",author,"}",sep="")
	}
	else{
		content[content=="@author"] <- "\\author{unknown author}"
	}
	if(!is.null(includes)){
		inc <- NULL
		for(i in 1:length(includes)){
			inc <- paste(inc,"\\include{",includes[i],"}\n",sep="")
		}
		content[content=="@includes"] <- inc
	}
	else{
		content[content=="@includes"] <- ""
	}	
	write(content,file=file)
	cat("Everything worked fine... \n")
}


###########
####
progress <- function(percent=0,steps=20,type="default",label=TRUE){
	switch(type,
		default={
		prog <- rep("#",steps)
		prog[2:(length(prog)-1)] <- " "
		prog[1:(floor(percent/100 * steps))] <- "#"
		if(percent!=0){
			if(label){
				back <- rep("\b",(steps+5))
			}
			else{
				back <- rep("\b",steps)
			}
			cat(back,sep="")
		}
		cat(prog,sep="")
		if(label){
			cat(rep(" ",3-length(floor(percent))),floor(percent),"%",sep="")
		}
		},
		arrow={
		arrowProgress(percent,steps)
		})
}

arrowProgress <- function(percent,steps=20,label=TRUE){
	prog <- rep("|",steps)
	prog[2:(length(prog)-1)] <- " "
	prog[2:(floor(percent/100 * steps))] <- "="
	prog[(floor(percent/100 * steps))] <- ">"
	if(percent!=0){
		if(label){
			back <- rep("\b",(steps+5))
		}
		else{
			back <- rep("\b",steps)
		}
		cat(back,sep="")
	}
	cat(prog,sep="")
	if(label){
		cat(rep(" ",3-length(floor(percent))),floor(percent),"%",sep="")
	}
}



##
##multiple apply, apply a function to two arrays (if the function needs two input parameters...)
multiapply <- function(a,b,MARGIN=NULL,FUN,...){
	if(is.null(MARGIN)){
		stop("Please submit the MARGIN: 1 for applying the function to the rows, 2 for applying them on columns!\n\n")
	}
	FUN <- match.fun(FUN)
	if(is.null(dim(a))){
		a <- as.matrix(a)
	}
	if(is.null(dim(b))){
		b <- as.matrix(b)
	}	
	if(MARGIN==1){
		a.length <- length(a[,1])
		if(a.length!=length(b[,1])){
			stop("The arrays a and b should have the same number of rows!\n\n")
		}
		test <- FUN(a[1,],b[1,],...)
		## if the result of the function is a character or an number we can create a result table
		if(class(test)=="numeric" | class(test)=="character"){
			result <- matrix(ncol=length(test),nrow=a.length)
			colnames(result) <- colnames(test)
			for(i in 1:a.length){
				result[i,] <- FUN(a[i,],b[i,],...)
			}
		}
		## create a list of the results... lists are cool
		else{
			result <- list()
			for(i in 1:a.length){
				result <- c(result,list(FUN(a[i,],b[i,],...)))
			}
		}
	}
	if(MARGIN==2){
		a.width <- length(a[1,])
		if(a.width!=length(b[1,])){
			stop("The arrays a and b should have the same number of cols!\n\n")
		}
		test <- FUN(a[,1],b[,1],...)
		## if the result of the function is a character or an number we can create a result table
		if(class(test)=="numeric" | class(test)=="character"){
			result <- matrix(nrow=length(test),ncol=a.width)
			rownames(result) <- colnames(test)
			for(i in 1:a.width){
				result[,i] <- FUN(a[,i],b[,i],...)
			}
		}
		## create a list of the results... lists are cool
		else{
			result <- list()
			for(i in 1:a.width){
				result <- c(result,list(FUN(a[,i],b[,i],...)))
			}
		}
	}
	result
}

mean.center <- function(x,log.scale=TRUE,sd.scale=FALSE,...){
	if(!log.scale){
		x <- log2(x)
	}
	mean.x <- mean(x,...)
	result <- x - mean.x
	if(sd.scale){
		sd.x <- sd(x,...)
		result <- result / sd.x
	}
	result	
}

openMadbVignette <- function(){
	f <- system.file("doc","maDB.pdf",package="maDB")
	if(.Platform$OS.type=="windows"){
		shell.exec(f)
	}
	else{
		system(paste(Sys.getenv("R_PDFVIEWER"),f,"&"))
	}
	return(f)
}



