\name{publishToDB}
\alias{publishToDB}
\title{Writing the expression values from normalized micro arrays into a database}
\description{
  \code{publishToDB} allows to store a microarray experiment (dataset) into a database. 
\\
For further informations and examples refer to the package vignette (which can be opened using the \code{\link{openMadbVignette}}).
}
\usage{
publishToDB(object=NULL,connection=NULL,exp.name=NULL,exp.date=NULL,samples=NULL,signal.channels=NULL,preprocessing=NULL,pubmed.id=NULL,global.transaction=TRUE,v=TRUE)
}
\arguments{
  \item{object}{The object from which the expression values (preprocessed intensity values) should be stored in the database (an object from the type \code{MadbSet}).}
  \item{connection}{The database connection, with the \code{dbConnect} function from the \pkg{RdbiPgSQL} package.}
  \item{exp.name}{The experiment name (project name or dataset name). This name has to be unique in the database!}
  \item{exp.date}{The date of the experiment. Optional; as additional information the publishing date will automoatically be inserted in the database.}
  \item{samples}{A list of, or a single representation of a \code{Sample} object, that contains information about the samples hybridized on the chips. If the sample already exists in the database, it will not be added again. This parameter can also be omitted (but then no sample information will be available for this array!)}
  \item{signal.channels}{The linking element between the signal channels (columns in the \code{exprs} slot of the \code{MadbSet} object), and the corresponding samples hybridized on the specific signal channel. A list of \code{SignalChannels} of a \code{MadbSet} is returned by calling the function \code{\link{getSignalChannels}} on the object. If a list of \code{Samples} are submitted, the linkage between the signal channels and the samples has to be genereated by setting the \code{sample.index} slot of the \code{SignalChannels} appropriately.}
  \item{preprocessing}{A string giving information about the preprocessing of the microarray experiment (e.g. RMA).}
  \item{pubmed.id}{Optional pubmed id of the microarray experiment (if it has already been published).}
  \item{global.transaction}{If a global transaction should be used for the whole funciton call. The benefit of this is, that if any errors occurr during the function call, no data will be written into the database. It is always a good idea to send a transaction rollback call to the database (e.g. dbSendQuery(Con,"ROLLBACK TRANSACTION;")) if any exceptions occurred, to be shure that the database will not get inconsitent.}
  \item{v}{If TRUE additional informations will be printed to the console.}
}
\details{
}
\references{}
\author{Johannes Rainer}
\seealso{
	\code{\link{getSignalChannels}}
	\code{\link{MadbSet-class}}
	\code{\link{SignalChannel-class}}
	\code{\link{Sample-class}}
	\code{\link{getPK}}
	\code{\link{loadFromDB}}
}

\examples{

}

\keyword{methods}

